@@ -371,7 +371,7 @@ rc_t KTocEntryNewChunked ( KTocEntry ** new_entry,
      *
      * Then add to that the size in bytes of the chunked data (all 64 bit numbers).
      */
-    nsize = ~(sizeof(uint64_t)-1) & 
+    nsize = ~( ( size_t ) sizeof(uint64_t)-1) & 
 	(sizeof(KTocEntry)
 	 - sizeof(union KTocEntryUnion)
 	 + sizeof(struct KTocEntryChunkFile)
@@ -1378,10 +1378,14 @@ bool check_limit (const void * ptr, const void * limit, size_t size)
 	ptr = *_ptr;							\
 									\
 	if (rev)							\
-	    *pout = S (*ptr++);						\
+	{								\
+	    T t;							\
+	    memcpy (&t, ptr, sizeof (T));				\
+	    *pout = S (t);						\
+	}								\
 	else								\
-	    *pout = *ptr++;						\
-	*_ptr = ptr;							\
+	    memcpy (pout, ptr, sizeof (T));                             \
+	*_ptr = ++ptr;							\
 	return 0;							\
     }
 
@@ -1445,6 +1449,7 @@ rc_t KTocEntryInflateNodeCommon (const void ** ptr,
 	memcpy (common->name+plen+1, *ptr, nlen);
 	common->name[plen + nlen + 1] = '\0';
     }
+
     *ptr = ((uint8_t*)*ptr) + nlen;
 
     rc = read_i64 (ptr, limit, rev, &common->mtime);
@@ -1717,27 +1722,23 @@ rc_t KTocInflatePBSTree ( KToc * self, uint64_t arcsize, const void * treestart,
     rc_t rc;
     PBSTree * pbst;
 
-    /* TBD - need to make this know about byte-swapping */
-    bool byteswap = false;
-
-    rc = PBSTreeMake (&pbst, treestart, maxsize, byteswap);
+    rc = PBSTreeMake (&pbst, treestart, maxsize, rev);
     if (rc == 0)
     {
-	KTocEntryInflateData data;
+        KTocEntryInflateData data;
 
-	data.toc = self;
-	data.path = path;
+        data.toc = self;
+        data.path = path;
         data.arcsize = arcsize;
-	data.rc = 0;
-	data.rev = rev;
-	data.offset = offset;
+        data.rc = 0;
+        data.rev = rev;
+        data.offset = offset;
+        
+        PBSTreeForEach (pbst, false, KTocEntryInflate, &data);
 
-	PBSTreeForEach (pbst, false, KTocEntryInflate, &data);
+        rc = data.rc;
 
-	rc = data.rc;
-
-
-        free (pbst);
+        PBSTreeWhack (pbst);
     }
     return rc;
 }
