@@ -43,10 +43,12 @@ struct KSysDir;
 #include <klib/log.h>
 #include <klib/out.h>
 #include <klib/debug.h>
+#include <klib/klib-priv.h>
 
 #include <sysalloc.h>
 
 #include <stdio.h>
+#include <wchar.h>
 #include <stdarg.h>
 #include <assert.h>
 #include <WINDOWS.H>
@@ -58,6 +60,156 @@ struct KSysDir;
 #define IO_REPARSE_TAG_SYMLINK 0xA000000C
 #endif
 
+
+/* Missing functions from our text library
+ * size is bytes; max_chars is number of elements
+ */
+
+/* utf16_utf32
+ *  converts UTF16 text to a single UTF32 character
+ *  returns the number of UTF16 words consumed, such that:
+ *    return > 0 means success
+ *    return == 0 means insufficient input
+ *    return < 0 means bad input or bad argument
+ */
+static
+int utf16_utf32 ( uint32_t *dst, const wchar_t *begin, const wchar_t *end )
+{
+    uint32_t ch;
+    const wchar_t *src, *stop;
+
+    if ( dst == NULL || begin == NULL || end == NULL )
+        return -1;
+
+    if ( begin == end )
+        return 0;
+
+    /* windows utf16 */
+
+    ch = (uint32_t)(begin [0]);
+
+    if ((ch < 0xD800) || (ch <= 0xE000))
+    {
+        *dst = ch;
+        return 1;
+    }
+    else
+    {
+        uint32_t ch;
+
+        /* need at least 2 words */
+        if (begin >= end)
+            return -1;
+
+        /* extreme checks */
+        if (((begin[0] & 0xFC00) != 0xD8) ||
+            ((begin[1] & 0xFC00) != 0xDC))
+            return -1;
+
+        ch = (begin[0] & 0x03FF) << 10 |
+            (begin[1] & 0x03FF);
+        return 2;
+    }
+}
+
+
+/* utf32_utf16
+ *  converts a single UTF32 character to UTF16 text
+ *  returns the number of UTF16 words generated, such that:
+ *    return > 0 means success
+ *    return == 0 means insufficient output
+ *    return < 0 means bad character or bad argument
+ */
+static 
+int utf32_utf16 ( wchar_t *begin, wchar_t *end, uint32_t ch )
+{
+    if (ch < 0x10000)
+    {
+        if ((ch <= 0xDFFF) && (ch >= 0xD800))
+            return -1;
+
+        begin[0] = (uint16_t)ch;
+        return 1;
+    }
+    else if ((ch >= 0x10FFFF) || (end <= begin))
+        return -1;
+    else
+    {
+        uint32_t cch;
+
+        cch = ch - 0x10000;
+        /* cch <= 0xFFFFF since ch < 0x10FFFF */
+
+        begin[0] = 0xD800 | (cch >> 10); /* upper 10 bits */
+        begin[1] = 0xDC00 | (cch & 0x3FF); /* lower 10 bita */
+        return 2;
+    }
+}
+
+
+static int wstrcase_cmp (const wchar_t * a, size_t asize,
+                         const wchar_t * b, size_t bsize,
+                         uint32_t max_chars)
+{
+    uint32_t num_chars;
+    const wchar_t *aend, *bend;
+
+    assert ( a != NULL && b != NULL );
+
+    /* set up end limit triggers */
+    aend = a + asize;
+    bend = b + bsize;
+
+    num_chars = 0;
+
+    while ( a < aend && b < bend )
+    {
+        uint32_t ach, bch;
+
+        /* read a character from a */
+        int len = utf16_utf32 ( & ach, a, aend );
+        if ( len <= 0 )
+        {
+            asize -= ( size_t ) ( aend - a );
+            break;
+        }
+        a += len;
+
+        /* read a character from b */
+        len = utf16_utf32 ( & bch, b, bend );
+        if ( len <= 0 )
+        {
+            bsize -= ( size_t ) ( bend - b );
+            break;
+        }
+        b += len;
+
+        /* compare characters with case */
+        if ( ach != bch )
+        {
+            /* only go lower case if they differ */
+            ach = towlower ( ( wint_t ) ach );
+            bch = towlower ( ( wint_t ) bch );
+
+            if ( ach != bch )
+            {
+                if ( ach < bch )
+                    return -1;
+                return 1;
+            }
+        }
+
+        /* if char count is sufficient, we're done */
+        if ( ++ num_chars == max_chars )
+            return 0;
+    }
+
+    /* one or both reached end < max_chars */
+    if (asize < bsize)
+        return -1;
+    return asize > bsize;
+}
+
 /*--------------------------------------------------------------------------
  * KSysDirEnum
  *  a Windows directory enumerator
@@ -107,8 +259,8 @@ rc_t KSysDirEnumInit ( KSysDirEnum *self, const wchar_t *path )
         rc = RC ( rcFS, rcDirectory, rcListing, rcNoObj, rcUnknown );
     }
 
-    PLOGERR ( klogErr,
-             ( klogErr, rc, "error FindWirstFileW - $(E) - $(C)",
+    PLOGERR ( klogInfo,
+             ( klogInfo, rc, "error FindFirstFileW - $(E) - $(C)",
               "E=%!,C=%u", err, err ) ); 
 
     return rc;
@@ -174,7 +326,13 @@ typedef VNamelist KSysDirListing;
 static
 int KSysDirListingSort ( const void *a, const void *b )
 {
-    return strcmp ( * ( const char** ) a, * ( const char** ) b );
+    size_t A,B,M;
+    A = wchar_string_size (a);
+    B = wchar_string_size (b);
+    /* close enough for max chars? */
+    M = (A>B) ? A : B;
+
+    return wstrcase_cmp (a, A, b, B, M);
 }
 
 static
@@ -279,6 +437,35 @@ rc_t translate_file_error( DWORD error, enum RCContext ctx )
 }
 
 
+/* helper */
+
+static rc_t print_error_for( const wchar_t * path, const char * function, enum RCContext ctx, KLogLevel level )
+{
+    DWORD error = GetLastError();
+    rc_t rc = translate_file_error( error, ctx );
+#if _DEBUGGING
+    char buffer[ 4096 ];
+    size_t src_size, dst_size, len;
+    wchar_cvt_string_measure ( path, &src_size, &dst_size );
+    len = wchar_cvt_string_copy ( buffer, sizeof buffer, path, src_size );
+    buffer[ len ] = 0;
+    PLOGERR ( level,
+              ( level, rc, "error $(F) - $(E) - $(C) for $(D)",
+                "F=%s,E=%!,C=%u,D=%s", function, error, error, buffer ) ); 
+#endif
+    return rc;
+}
+
+
+static void wchar_2_char( const wchar_t * path, char * buffer, size_t buflen )
+{
+    size_t src_size, dst_size, len;
+    wchar_cvt_string_measure ( path, &src_size, &dst_size );
+    len = wchar_cvt_string_copy ( buffer, buflen, path, src_size );
+    buffer[ len ] = 0;
+}
+
+
 static
 uint32_t KSysDirPathTypeFromFindData ( WIN32_FIND_DATA *find_data, 
                                        const wchar_t * path,
@@ -381,9 +568,7 @@ uint32_t KSysDirFullPathType ( const wchar_t *path )
             return KSysDirResolvePathAndDetectPathType ( path );
         default:
             DBGMSG ( DBG_KFS, DBG_FLAG_ANY, ( "FindFirstFileW: WARNING - unrecognized return code - %u.\n", status ) );
-            PLOGERR ( klogErr,
-                     ( klogErr, 0, "error FindWirstFileW - $(E) - $(C)",
-                      "E=%!,C=%u", status, status ) ); 
+            print_error_for( path, "FindFirstFileW", rcResolving, klogErr );
             return kptBadPath;
         }
     }
@@ -559,7 +744,7 @@ rc_t KSysDirMakePath ( const KSysDir* self, enum RCContext ctx, bool canon,
                 return RC ( rcFS, rcDirectory, ctx, rcPath, rcInvalid );
 
             /* rewrite drive letter */
-            temp_utf8_buffer [ 0 ] = toupper ( temp_utf8_buffer [ 1 ] );
+            temp_utf8_buffer [ 0 ] = tolower ( temp_utf8_buffer [ 1 ] );
             temp_utf8_buffer [ 1 ] = ':';
         }
     }
@@ -616,7 +801,7 @@ rc_t KSysDirMakePath ( const KSysDir* self, enum RCContext ctx, bool canon,
     return 0;
 }
 
-rc_t KSysDirRealPath ( const KSysDir *self,
+LIB_EXPORT rc_t KSysDirOSPath ( const KSysDir *self,
     wchar_t *real, size_t real_size, const char *path, va_list args )
 {
     return KSysDirMakePath ( self, rcLoading, true, real, real_size, path, args );
@@ -871,27 +1056,19 @@ rc_t KSysDirRelativePath ( const KSysDir *self, enum RCContext ctx,
     const wchar_t *root, wchar_t *path, size_t path_max )
 {
     size_t psize;
-    uint32_t backup, blength_in_chars;
+    uint32_t backup, blength_in_chars, dst, diff_from_here;
 
     const wchar_t *r = root + self->root;
     const wchar_t *p = path + self->root;
 
-    for ( ; * r == * p; ++ r, ++ p )
+    /* stop gap fix..  not actually comparing the utf16 values correctly */
+    for ( ; towlower (*r) == towlower (*p); ++ r, ++ p )
     {
         /* disallow identical paths */
         if ( * r == 0 )
             return RC( rcFS, rcDirectory, ctx, rcPath, rcInvalid );
     }
 
-    /* special case for windows: a network path like "\\panfs\traces01\" 
-       cannot be made relative to a working directory ! */
-
-    /* NB - THE BUG HERE IS THAT THE TWO PATHS MUST BE ON DIFFERENT FILE SYSTEMS */
-    if ( ( path[0] == '\\' ) && ( path[1] == '\\' ) )
-    {
-/*        return 0; */
-    }
-
     /* paths are identical up to "r","p"
        if "r" is within a leaf name, then no backup is needed
        by counting every '\\' from "r" to end, obtain backup count */
@@ -905,31 +1082,35 @@ rc_t KSysDirRelativePath ( const KSysDir *self, enum RCContext ctx,
     blength_in_chars = backup * 3;
 
     /* align "p" to last directory separator */
-    while ( p [ -1 ] != '\\' ) -- p;
+    if ( p > path ) {
+        while ( p [ -1 ] != '\\' ) -- p;
+    }
 
     /* the size of the remaining relative path */
     psize = wcslen ( p );
+    diff_from_here = ( uint32_t )( p - path );
 
     /* open up space if needed */
-    if ( (uint32_t)( p - path ) < blength_in_chars )
+    if ( diff_from_here < blength_in_chars )
     {
         /* prevent overflow */
         if ( ( blength_in_chars + psize ) * sizeof( *path ) >= path_max )
             return RC( rcFS, rcDirectory, ctx, rcPath, rcExcessive );
-        memmove ( path + blength_in_chars, p, psize * sizeof *p );
+        memmove ( & path[ blength_in_chars ], p, psize * ( sizeof *p ) );
     }
 
     /* insert backup sequences */
-    for ( blength_in_chars = 0; backup > 0; blength_in_chars += 3, -- backup )
+    for ( dst = 0; backup > 0; -- backup )
     {
-        path [ blength_in_chars + 0 ] = '.';
-        path [ blength_in_chars + 1 ] = '.';
-        path [ blength_in_chars + 2 ] = '\\';
+        path [ dst++ ] = '.';
+        path [ dst++ ] = '.';
+        path [ dst++ ] = '\\';
     }
 
     /* close gap */
-    if ( ( uint32_t) ( p - path ) > blength_in_chars )
+    if ( diff_from_here > blength_in_chars )
         wcscpy ( & path [ blength_in_chars ], p );
+    path[ blength_in_chars + psize ] = 0;
 
     return 0;
 }
@@ -957,10 +1138,15 @@ rc_t CC KSysDirResolvePath ( const KSysDir *self, bool absolute,
     uint32_t temp_length;
 
     /* convert the utf8-input-parameter path into wchar_t */
-    rc_t rc = KSysDirMakePath( self, rcResolving, true, temp, sizeof temp, path, args);
+    rc_t rc = KSysDirMakePath ( self, rcResolving, true, temp, sizeof temp, path, args );
     if ( rc != 0 )
         return rc;
 
+    temp[ 0 ] = tolower( temp[ 0 ] ); /* this is important:
+                                         otherwise the comparison for is_on_same_drive_letter fails
+                                         AND
+                                         KSysDirRelativePath() fails too! */
+
     temp_length = wchar_string_measure ( temp, &temp_size );
     if ( absolute )
     {
@@ -970,11 +1156,26 @@ rc_t CC KSysDirResolvePath ( const KSysDir *self, bool absolute,
     }
     else
     {
-        rc = KSysDirRelativePath( self, rcResolving, self->path, temp, sizeof temp /* temp_size */ );
-        if ( rc == 0 )
+        /* we are on windows, only if the path has a drive letter and it is the same
+           one as in KSysDir itself, we should try to create a relative path */
+        wchar_t colon = ':';
+        bool is_on_same_drive_letter = ( iswascii ( temp[ 0 ] ) && iswascii ( self->path[ 0 ] ) &&
+                                         ( temp[ 1 ] == colon ) && ( self->path[ 1 ] == colon ) &&
+                                         ( towlower ( temp[ 0 ] ) == towlower ( self->path[ 0 ] ) ) );
+        if ( is_on_same_drive_letter )
         {
-            uint32_t temp_length = wchar_string_measure ( temp, &temp_size );
-            if ( temp_length >= rsize )
+            rc = KSysDirRelativePath( self, rcResolving, self->path, temp, sizeof temp );
+            if ( rc == 0 )
+            {
+                uint32_t temp_length = wchar_string_measure ( temp, &temp_size );
+                if ( temp_length >= rsize )
+                    return RC ( rcFS, rcDirectory, rcResolving, rcBuffer, rcInsufficient );
+            }
+        }
+        else
+        {
+            /* treat it as if absolute were requested ( see above ) */
+            if ( temp_length - self->root >= rsize )
                 return RC ( rcFS, rcDirectory, rcResolving, rcBuffer, rcInsufficient );
         }
     }
@@ -1074,7 +1275,8 @@ rc_t CC KSysDirResolveAlias ( const KSysDir *self, bool absolute,
         {
             /* the path in full is an absolute path
                if outside of chroot, it's a bad link */
-            if ( memcmp( temp.path, self->path, self->root + 1 ) != 0 )
+            if (wstrcase_cmp (temp.path, self->root + 1,
+                              self->path, self->root + 1,self->root + 1) != 0)
                 return RC( rcFS, rcDirectory, rcResolving, rcLink, rcInvalid );
 
             /* this is the absolute path length */
@@ -1178,15 +1380,19 @@ rc_t CC KSysDirResolveAlias ( const KSysDir *self, bool absolute,
         rc = KSysDirCanonPath( &full, rcResolving, full.path, len );
         if ( rc == 0 )
         {
+            size_t f, s;
+
+            f = wchar_string_size (full.path);
+            s = wchar_string_size (self->path);
             /* the path in full is an absolute path
                if outside of chroot, it's a bad link */
-            if ( memcmp( full.path, self->path, self->root + 1 ) != 0 )
+            if ( wstrcase_cmp (full.path, f, self->path, s, self->root + 1 ) != 0 )
             {
                 return RC( rcFS, rcDirectory, rcResolving, rcLink, rcInvalid );
             }
 
             /* this is the absolute path length */
-            len = strlen( &full.path[self->root] );
+            len = wchar_string_size( &full.path[self->root] );
 
             /* if not requesting absolute, make self relative */
             if( !absolute )
@@ -1194,12 +1400,12 @@ rc_t CC KSysDirResolveAlias ( const KSysDir *self, bool absolute,
             rc = KSysDirRelativePath( self, rcResolving, self->path, full.path, sizeof full.path /*len*/ );
                 if ( rc != 0 )
                     return rc;
-                len = strlen(full.path);
+                len = wchar_string_size(full.path);
             }
             if ( ( size_t ) len >= rsize )
                 return RC(rcFS, rcDirectory, rcResolving, rcBuffer, rcInsufficient );
 
-            strcpy ( resolved, & full . path [ self -> root ] );
+            wcscpy ( resolved, & full . path [ self -> root ] );
         }
 #endif
     return rc;
@@ -1222,15 +1428,25 @@ rc_t CC KSysDirRename ( KSysDir *self, bool force, const char *from, const char
     rc_t rc = KSysDirMakePath ( self, rcRenaming, false, current_name, sizeof current_name, from, NULL );
     if ( rc == 0 )
     {
+
         wchar_t new_name[ MAX_PATH ];
         rc = KSysDirMakePath ( self, rcRenaming, false, new_name, sizeof new_name, to, NULL );
         if ( rc == 0 )
         {
             DWORD err = 0;
             uint32_t try = 0;
+
             do
             {
-                if ( MoveFileW( current_name, new_name ) )
+                BOOL success = false;
+                if ( force ) {
+                    DWORD dwFlags = MOVEFILE_REPLACE_EXISTING;
+                    success = MoveFileEx ( current_name, new_name, dwFlags );
+                }
+                else {
+                    success = MoveFileW ( current_name, new_name );
+                }
+                if ( success )
                 {
                     rc = 0;
                 }
@@ -1248,6 +1464,8 @@ rc_t CC KSysDirRename ( KSysDir *self, bool force, const char *from, const char
                             Sleep( 500 ); /* sleep for a half a second */
                             rc = RC ( rcFS, rcDirectory, rcRenaming, rcFile, rcUnauthorized );
                             break;
+                       case ERROR_SHARING_VIOLATION:
+                           return RC ( rcFS, rcDirectory, rcRenaming, rcFile, rcBusy );
                         default:
                             return RC ( rcFS, rcDirectory, rcRenaming, rcNoObj, rcUnknown );
                     }
@@ -1432,9 +1650,7 @@ rc_t KSysDirRemoveEntry ( wchar_t *path, size_t path_max, bool force )
                         if ( !RemoveDirectoryW( path ) )
                         {
                             rc = RC ( rcFS, rcDirectory, rcRemoving, rcDirectory, rcUnauthorized );
-                            PLOGERR ( klogErr,
-                                      ( klogErr, rc, "error RemoveDirectoryW - $(E) - $(C)",
-                                       "E=%!,C=%u", error, error ) ); 
+                            print_error_for( path, "RemoveDirectoryW", rcRemoving, klogErr );
                         }
                     }
                     return rc;
@@ -1448,9 +1664,8 @@ rc_t KSysDirRemoveEntry ( wchar_t *path, size_t path_max, bool force )
             default :
                 rc = RC ( rcFS, rcDirectory, rcCreating, rcNoObj, rcUnknown );
             }
-            PLOGERR ( klogErr,
-                      ( klogErr, rc, "error RemoveDirectoryW - $(E) - $(C)",
-                       "E=%!,C=%u", error, error ) ); 
+
+            print_error_for( path, "RemoveDirectoryW", rcRemoving, klogInfo);
             return rc;
         }
     }
@@ -1534,11 +1749,8 @@ rc_t get_attributes ( const wchar_t * wpath, uint32_t * access, KTime_t * date )
         *access = 0;
     if ( date != NULL )
         *date = 0;
-    error = GetLastError();
-    rc = translate_file_error( error, rcAccessing );
-    PLOGERR ( klogErr,
-              ( klogErr, rc, "error FindFirstFile - $(E) - $(C)",
-               "E=%!,C=%u", error, error ) ); 
+
+    rc = print_error_for( wpath, "FindFirstFile", rcAccessing, klogErr );
     return rc;
 }
 
@@ -1751,12 +1963,10 @@ rc_t make_dir ( const wchar_t *path, uint32_t access )
     {
         DWORD error = GetLastError();
         rc = translate_file_error( error, rcCreating );
-        if ( error != ERROR_ALREADY_EXISTS )
-        {
-            PLOGERR ( klogErr,
-                      ( klogErr, rc, "error CreateDirectoryW - $(E) - $(C)",
-                        "E=%!,C=%u", error, error ) );
-        }
+/*
+        Do not print an error code here, it is valid that this can happen!
+        rc = print_error_for( path, "CreateDirectoryW", rcCreating, klogErr );
+*/
     }
     return rc;
 }
@@ -1960,19 +2170,17 @@ rc_t CC KSysDirOpenFileRead ( const KSysDir *self,
     rc_t rc = KSysDirMakePath( self, rcOpening, false, file_name, sizeof file_name, path, args );
     if ( rc == 0 )
     {
-        HANDLE file_handle = CreateFileW( file_name, GENERIC_READ, FILE_SHARE_READ, NULL, 
+        HANDLE file_handle = CreateFileW( file_name, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, 
                                 OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
         if ( file_handle == INVALID_HANDLE_VALUE )
         {
-            DWORD error = GetLastError();
-            rc = translate_file_error( error, rcOpening );
-            PLOGERR ( klogErr,
-                      ( klogErr, rc, "error CreateFileW - $(E) - $(C)",
-                        "E=%!,C=%u", error, error ) ); 
+            rc = print_error_for( file_name, "CreateFileW", rcOpening, klogInfo );
         }
         else
         {
-            rc = KSysFileMake ( ( KSysFile** ) f, file_handle, true, false );
+            char buffer[ MAX_PATH ];
+            wchar_2_char( file_name, buffer, sizeof buffer );
+            rc = KSysFileMake ( ( KSysFile** ) f, file_handle, buffer, true, false );
             if ( rc != 0 )
                 CloseHandle ( file_handle );
         }
@@ -2000,20 +2208,19 @@ rc_t CC KSysDirOpenFileWrite ( KSysDir *self,
     if ( rc == 0 )
     {
         DWORD dwDesiredAccess = update ? GENERIC_READ | GENERIC_WRITE : GENERIC_WRITE;
-        HANDLE file_handle = CreateFileW( file_name, dwDesiredAccess, 0, NULL, 
-                                OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
+        HANDLE file_handle = CreateFileW( file_name, dwDesiredAccess, FILE_SHARE_READ, NULL, 
+                                OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL );
 
         if ( file_handle == INVALID_HANDLE_VALUE )
         {
-            DWORD error = GetLastError();
-            rc = translate_file_error( error, rcAccessing );
-            PLOGERR ( klogErr,
-                      ( klogErr, rc, "error CreateFileW - $(E) - $(C)",
-                        "E=%!,C=%u", error, error ) ); 
+            rc = print_error_for( file_name, "CreateFileW", rcAccessing, klogErr );
+
         }
         else
         {
-            rc = KSysFileMake ( ( KSysFile** ) f, file_handle, update, true );
+            char buffer[ MAX_PATH ];
+            wchar_2_char( file_name, buffer, sizeof buffer );
+            rc = KSysFileMake ( ( KSysFile** ) f, file_handle, buffer, update, true );
             if ( rc != 0 )
                 CloseHandle ( file_handle );
         }
@@ -2085,17 +2292,29 @@ rc_t CC KSysDirCreateFile ( KSysDir *self, KFile **f, bool update,
 
             error = GetLastError();
             rc = translate_file_error( error, rcCreating );
+
+            /* disabled 12/12/2012 : it prints an error message, if vdb tries to open
+               the same reference-object twice via http. The lock-file for the 2nd try
+               does already exist. This is not an error, just a condition. */
+
+            /*
             PLOGERR ( klogErr,
                       ( klogErr, rc, "error CreateFileW - $(E) - $(C)",
                         "E=%!,C=%u", error, error ) ); 
+            */
+
             /* Unix code has a special case when creating an empty file, which is
                to say, creating a directory entry without needing to write to file */
             return rc;
         }
 
-        rc = KSysFileMake ( ( KSysFile** ) f, file_handle, update, true );
-        if ( rc != 0 )
-            CloseHandle ( file_handle );
+        {
+            char buffer[ MAX_PATH ];
+            wchar_2_char( file_name, buffer, sizeof buffer );
+            rc = KSysFileMake ( ( KSysFile** ) f, file_handle, path, update, true );
+            if ( rc != 0 )
+                CloseHandle ( file_handle );
+        }
     }
     return rc;
 }
@@ -2125,11 +2344,7 @@ rc_t CC KSysDirFileSize ( const KSysDir *self,
         }
         else
         {
-            DWORD error = GetLastError();
-            rc = translate_file_error( error, rcAccessing );
-            PLOGERR ( klogErr,
-                      ( klogErr, rc, "error GetFileAttributesEx - $(E) - $(C)",
-                        "E=%!,C=%u", error, error ) ); 
+            rc = print_error_for( file_name, "GetFileAttributesEx", rcAccessing, klogErr );
         }
     }
     return rc;
@@ -2172,11 +2387,7 @@ rc_t CC KSysDirSetFileSize ( KSysDir *self,
         }
         else
         {
-            DWORD error = GetLastError();
-            rc = translate_file_error( error, rcUpdating );
-            PLOGERR ( klogErr,
-                      ( klogErr, rc, "error CreateFileW - $(E) - $(C)",
-                        "E=%!,C=%u", error, error ) ); 
+            rc = print_error_for( file_name, "CreateFileW", rcUpdating, klogErr );
         }
     }
     return rc;
@@ -2371,11 +2582,7 @@ rc_t change_item_date( wchar_t *path, LPFILETIME win_time, bool dir_flag )
                                    OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL );
     if ( file_handle == INVALID_HANDLE_VALUE )
     {
-        DWORD error = GetLastError();
-        rc = translate_file_error( error, rcUpdating );
-        PLOGERR ( klogErr,
-                  ( klogErr, rc, "error CreateFileW - $(E) - $(C)",
-                    "E=%!,C=%u", error, error ) ); 
+        rc = print_error_for( path, "CreateFileW", rcUpdating, klogErr );
     }
     else
     {
@@ -2385,11 +2592,7 @@ rc_t change_item_date( wchar_t *path, LPFILETIME win_time, bool dir_flag )
         }
         else
         {
-            DWORD error = GetLastError();
-            rc = translate_file_error( error, rcUpdating );
-            PLOGERR ( klogErr,
-                      ( klogErr, rc, "error SetFileTime - $(E) - $(C)",
-                        "E=%!,C=%u", error, error ) ); 
+            rc = print_error_for( path, "SetFileTime", rcUpdating, klogErr );
         }
         CloseHandle ( file_handle );
     }
@@ -2678,10 +2881,21 @@ rc_t KDirectoryMakeFromRealPath ( KDirectory **dirp, const wchar_t *real, bool u
  *
  *  "dir" [ OUT ] - return parameter for native directory
  */
+extern rc_t CC ReportCWD ( const ReportFuncs *f, uint32_t indent );
+extern rc_t CC ReportRedirect ( KWrtHandler* handler,
+    const char* filename, bool* to_file, bool finalize );
+
 LIB_EXPORT rc_t CC KDirectoryNativeDir ( KDirectory **dirp )
 {
     rc_t rc;
 
+    static bool latch;
+    if ( ! latch )
+    {
+        ReportInitKFS ( ReportCWD, ReportRedirect );
+        latch = true;
+    }
+
     if ( dirp == NULL )
         rc = RC ( rcFS, rcDirectory, rcAccessing, rcParam, rcNull );
     else
@@ -2710,3 +2924,31 @@ LIB_EXPORT rc_t CC KDirectoryNativeDir ( KDirectory **dirp )
 
     return rc;
 }
+
+
+/* RealPath
+ *  exposes functionality of system directory
+ */
+LIB_EXPORT rc_t CC KSysDirRealPath ( struct KSysDir const *self,
+    char *real, size_t bsize, const char *path, ... )
+{
+    rc_t rc;
+    va_list args;
+
+    va_start ( args, path );
+    rc = KSysDirVRealPath ( self, real, bsize, path, args );
+    va_end ( args );
+
+    return rc;
+}
+
+LIB_EXPORT rc_t CC KSysDirVRealPath ( struct KSysDir const *self,
+    char *real, size_t bsize, const char *path, va_list args )
+{
+    /* Windows is ... challenged when it comes to answering
+       this question. What is needed is to 1) convert the path
+       to a Windows-style wchar path, then 2) resolve each of
+       its components, etc. to come up with a real path, then
+       3) rewrite the path as a UTF-8 POSIX path */
+    return KSysDirResolvePath ( self, true, real, bsize, path, args );
+}
