@@ -24,6 +24,10 @@
 *
 */
 
+#ifndef _WIN32_WINNT /* This file requires OS newer than Windows 2000 */
+# define _WIN32_WINNT 0x0501
+#endif
+
 #include <kfs/extern.h>
 
 #define TRACK_REFERENCES 0
@@ -33,6 +37,7 @@
 
 #include <kfs/dyload.h>
 #include <kfs/directory.h>
+#include <kfs/kfs-priv.h>
 #include <klib/refcount.h>
 #include <klib/vector.h>
 #include <klib/text.h>
@@ -190,7 +195,7 @@ LIB_EXPORT rc_t CC KDyldRelease ( const KDyld *self )
         {
         case krefWhack:
             return KDyldWhack ( ( KDyld* ) self );
-        case krefLimit:
+        case krefNegative:
             return RC ( rcFS, rcDylib, rcReleasing, rcRange, rcExcessive );
         }
     }
@@ -224,7 +229,7 @@ rc_t KDyldSever ( const KDyld *self )
         {
         case krefWhack:
             return KDyldWhack ( ( KDyld* ) self );
-        case krefLimit:
+        case krefNegative:
             return RC ( rcFS, rcDylib, rcReleasing, rcRange, rcExcessive );
         }
     }
@@ -311,7 +316,7 @@ LIB_EXPORT rc_t CC KDyldHomeDirectory ( const KDyld *self, const KDirectory **di
              * a type safe way to do this */
             BOOL success = GetModuleHandleEx ( GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS
                                                | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT,
-                                               (LPCSTR)func, & h );
+                                               ( const TCHAR * )func, & h );
             if ( success )
             {
                 wchar_t fname [ MAX_PATH ];
@@ -474,6 +479,9 @@ static
 rc_t KDyldLoad ( KDyld *self, KDylib *lib, const wchar_t *path )
 {
     DWORD err;
+#if WE_WERE_BUILDING_FOR_WINDOWS_7_ALONE
+    UINT errMode = GetErrorMode();
+#endif
 
 	if ( path == NULL )
 	{
@@ -483,15 +491,21 @@ rc_t KDyldLoad ( KDyld *self, KDylib *lib, const wchar_t *path )
         return RC ( rcFS, rcDylib, rcLoading, rcNoObj, rcUnknown );
 	}
 
+    SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOOPENFILEERRORBOX); /* suppress the message box in case of an error */
     lib -> handle = LoadLibraryW ( path );
+#if WE_WERE_BUILDING_FOR_WINDOWS_7_ALONE
+    SetErrorMode(errMode);
+#endif
     if ( lib -> handle != NULL )
         return KDylibSetLogging ( lib );
 
     err = GetLastError ();
     switch ( err )
     {
-    case ERROR_MOD_NOT_FOUND:
+    case ERROR_MOD_NOT_FOUND :
         return RC ( rcFS, rcDylib, rcLoading, rcPath, rcNotFound );
+    case ERROR_BAD_EXE_FORMAT :
+        return RC ( rcFS, rcDylib, rcLoading, rcFormat, rcInvalid );
     }
 
     return RC ( rcFS, rcDylib, rcLoading, rcNoObj, rcUnknown );
@@ -510,7 +524,7 @@ rc_t KDyldVTryLoadLib ( KDyld *self, KDylib **lib,
     {
         wchar_t real [ MAX_PATH ];
 
-		rc = KSysDirRealPath ( sdir, real, sizeof real, path, args );
+		rc = KSysDirOSPath ( sdir, real, sizeof real, path, args );
 		if ( rc == 0 )
 		{
 			WString pstr;
@@ -665,7 +679,7 @@ LIB_EXPORT rc_t CC KDylibRelease ( const KDylib *self )
         {
         case krefWhack:
             return KDylibWhack ( ( KDylib* ) self );
-        case krefLimit:
+        case krefNegative:
             return RC ( rcFS, rcDylib, rcReleasing, rcRange, rcExcessive );
         }
     }
@@ -706,7 +720,7 @@ rc_t KDylibSever ( const KDylib *self )
         {
         case krefWhack:
             return KDylibWhack ( ( KDylib* ) self );
-        case krefLimit:
+        case krefNegative:
             return RC ( rcFS, rcDylib, rcReleasing, rcRange, rcExcessive );
         }
     }
@@ -797,7 +811,8 @@ rc_t KDlsetWhack ( KDlset *self )
     return 0;
 }
 
-
+#define STRINGIZE(s) #s
+#define LIBNAME(pref, name, suff) STRINGIZE(pref) name STRINGIZE(suff)
 /* MakeSet
  *  load a dynamic library
  *
@@ -805,8 +820,8 @@ rc_t KDlsetWhack ( KDlset *self )
  */
 LIB_EXPORT rc_t CC KDyldMakeSet ( const KDyld *self, KDlset **setp )
 {
-    rc_t rc;
-
+    rc_t rc = 0;
+    
     if ( setp == NULL )
         rc = RC ( rcFS, rcDylib, rcConstructing, rcParam, rcNull );
     else
@@ -825,8 +840,21 @@ LIB_EXPORT rc_t CC KDyldMakeSet ( const KDyld *self, KDlset **setp )
                 VectorInit ( & set -> ord, 0, 16 );
                 KRefcountInit ( & set -> refcount, 1, "KDlset", "make", "dlset" );
 #if ! ALWAYS_ADD_EXE
-                * setp = set;
-                return 0;
+                {   
+                    KDylib *jni;
+                    const char* libname = "vdb_jni.dll";
+                    rc = KDyldLoadLib ( ( KDyld* ) self, & jni, libname );
+                    if ( rc == 0 )
+                    {
+                        rc = KDlsetAddLib ( set, jni );
+                        KDylibRelease ( jni );
+                    }
+                    /*if (rc == 0)*/ /* if JNI code is not there, C tools should not suffer */
+                    {
+                        * setp = set;
+                        return 0;
+                    }
+                }
 #else
                 {
                     KDylib *exe;
@@ -879,7 +907,7 @@ LIB_EXPORT rc_t CC KDlsetRelease ( const KDlset *self )
         {
         case krefWhack:
             return KDlsetWhack ( ( KDlset* ) self );
-        case krefLimit:
+        case krefNegative:
             return RC ( rcFS, rcDylib, rcReleasing, rcRange, rcExcessive );
         }
     }
@@ -900,7 +928,7 @@ rc_t KDlsetAddLibInt ( KDlset *self, KDylib *lib )
     if ( rc == 0 )
     {
         void *ignore;
-
+        
         rc = VectorInsertUnique ( & self -> name,
             lib, NULL, KDylibSort );
         if ( rc == 0 )
@@ -1043,7 +1071,7 @@ LIB_EXPORT rc_t CC KSymAddrRelease ( const KSymAddr *self )
         {
         case krefWhack:
             return KSymAddrWhack ( ( KSymAddr* ) self );
-        case krefLimit:
+        case krefNegative:
             return RC ( rcFS, rcDylib, rcReleasing, rcRange, rcExcessive );
         }
     }
@@ -1248,7 +1276,7 @@ LIB_EXPORT rc_t CC KDlsetLastSymbol ( const KDlset *self, KSymAddr **sym, const
     bool ( CC * test ) ( const KSymAddr *sym, void *data ), void *data )
 {
     rc_t rc;
-
+    
     if ( sym == NULL )
         rc = RC ( rcFS, rcDylib, rcSelecting, rcParam, rcNull );
     else
@@ -1288,6 +1316,80 @@ LIB_EXPORT rc_t CC KDlsetLastSymbol ( const KDlset *self, KSymAddr **sym, const
 }
 
 
+/* List - PRIVATE
+ *  list the paths to the libraries in the set
+ */
+typedef struct list_dylib_param list_dylib_param;
+struct list_dylib_param
+{
+    VNamelist *list;
+    rc_t rc;
+};
+
+static
+bool CC list_dylib ( void *item, void *data )
+{
+    size_t bytes;
+    char utf8 [ 4096 ], *p;
+
+    list_dylib_param *pb = data;
+    const KDylib *lib = ( const void* ) item;
+
+    /* "lib" was created with KDylibMake
+       which creates a NUL terminated path.
+       of course, this could seg-fault if bad... */
+    assert ( lib -> path . addr [ lib -> path . len ] == 0 );
+
+    /* convert wide-character to UTF-8 */
+    bytes = wchar_cvt_string_copy ( utf8, sizeof utf8, lib -> path . addr, lib -> path . size );
+    if ( bytes < sizeof utf8 )
+    {
+        pb -> rc = VNamelistAppend ( pb -> list, utf8 );
+        return pb -> rc != 0;
+    }
+
+    p = malloc ( 16 * 1024 );
+    if ( p == NULL )
+    {
+        pb -> rc = RC ( rcFS, rcDylib, rcListing, rcMemory, rcExhausted );
+        return true;
+    }
+
+    bytes = wchar_cvt_string_copy ( p, 16 * 1024, lib -> path . addr, lib -> path . size );
+    if ( bytes >= 16 * 1024 )
+        pb -> rc = RC ( rcFS, rcDylib, rcListing, rcPath, rcTooLong );
+    else
+        pb -> rc = VNamelistAppend ( pb -> list, p );
+
+    free ( p );
+    return pb -> rc != 0;
+}
+
+LIB_EXPORT rc_t CC KDlsetList ( const KDlset *self, KNamelist **listp )
+{
+    list_dylib_param pb;
+
+    assert ( listp != NULL );
+
+    if ( self == NULL )
+        pb . rc = RC ( rcFS, rcDylib, rcListing, rcSelf, rcNull );
+    else
+    {
+        pb . rc = VNamelistMake ( & pb . list, VectorLength ( & self -> name ) );
+        if ( pb . rc == 0 )
+        {
+            bool fail = VectorDoUntil ( & self -> name, false, list_dylib, & pb );
+            if ( ! fail )
+                pb . rc = VNamelistToNamelist ( pb . list, listp );
+
+            VNamelistRelease ( pb . list );
+        }
+    }
+
+    return pb . rc;
+}
+
+
 /* AsObj
  * AsFunc
  *  retrieve symbol address as pointer to object
