@@ -26,7 +26,7 @@
 
 #include <kfs/extern.h>
 
-#include <klib/xml.h>
+#include <kxml/xml.h>
 
 #include <klib/defs.h>
 #include <klib/container.h>
@@ -35,6 +35,7 @@
 #include <klib/refcount.h>
 #include <klib/vector.h>
 #include <klib/log.h>
+#include <klib/out.h>
 #include <klib/namelist.h>
 
 #include <kfs/file.h>
@@ -42,6 +43,11 @@
 #include <kfs/gzip.h>
 #include <kfs/directory.h>
 
+#include <vfs/manager.h>
+#include <vfs/path.h>
+
+#include <sysalloc.h>
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -115,7 +121,7 @@ struct XTocEntry
     XTocEntry_t type;           /* XTocEntry_e */
     union utype
     {
-        struct idtype
+        struct /* idtype - name conflict on Solaris */
         {
             XTocEntry * target;
         } id;
@@ -234,6 +240,7 @@ rc_t XTocEntryResolvePath (const XTocEntry * self, const char * path_, bool foll
     const char * path;
     const char * limit;
     XTocEntry * node;
+    size_t size;
 
     assert (self);
     assert (path_);
@@ -250,7 +257,7 @@ rc_t XTocEntryResolvePath (const XTocEntry * self, const char * path_, bool foll
     /* this was causing a seg fault for some reason */
     limit = path + string_size (path);
 #else
-    limit = path + strlen (path);
+    limit = path + string_measure(path, &size);
 #endif
 
     do
@@ -874,7 +881,7 @@ rc_t XTocMake (XToc ** pself, const String * base_path)
     KRefcountInit (&self->refcount, 1, XTocClassname, "Init", base_path->addr);
 
     rc = XTocEntryMakeInt (&self->root, "/", NULL, NULL,
-                           (KTime_t)time, 0);
+                           (KTime_t)time(NULL), 0);
     if (rc)
     {
         free (self);
@@ -1120,7 +1127,7 @@ static
         rc = RC (rcFS, rcFile, rcConstructing, rcMemory, rcExhausted);
     else
     {
-        rc = KFileInit (&self->dad, (const KFile_vt*)&vtKXTocFile, true, false);
+        rc = KFileInit (&self->dad, (const KFile_vt*)&vtKXTocFile, "KXTocFile", "no-name", true, false);
         if (rc == 0)
         {
             rc = KFileAddRef (base);
@@ -1163,14 +1170,6 @@ static
 }
 
 
-
-
-
-
-
-
-
-
 struct KXTocDir
 {
     KDirectory dad;
@@ -1180,7 +1179,7 @@ struct KXTocDir
     XTocEntry * entry;
     XTocEntry * container;
     XTocEntry * root;
-    String base_path;
+    String      base_path;
 };
 
 #if _DEBUGGING
@@ -1893,6 +1892,7 @@ rc_t KXTocDirRelativePath (const KXTocDir *self, enum RCContext ctx,
 #else
     int backup;
     size_t bsize, psize;
+    size_t size;
 
     const char *r = root + self -> root;
     const char *p = path + self -> root;
@@ -1923,7 +1923,7 @@ rc_t KXTocDirRelativePath (const KXTocDir *self, enum RCContext ctx,
     while (p [ -1 ] != '/') -- p;
 
     /* the size of the remaining relative path */
-    psize = strlen (p);
+    psize = string_measure(p, &size);
 
     /* open up space if needed */
     if ( (size_t)(p - path) < bsize )
@@ -1941,7 +1941,8 @@ rc_t KXTocDirRelativePath (const KXTocDir *self, enum RCContext ctx,
     /* close gap */
     if ( (size_t)( p - path ) > bsize )
 	{
-		strcpy (& path [ bsize ], p);
+        size_t size;
+		string_copy (& path [ bsize ], path_max - bsize, p, measure_string(p, &size));
 	}
 
 	return 0;
@@ -2034,13 +2035,14 @@ rc_t CC KXTocDirVisit (const KXTocDir *self,
 
             if (rc == 0)
             {
+                size_t size;
 		KXTocDir * full_dir;
 		uint32_t path_size;
 
 		/* -----
 		 * make a locally accessible private KDirectory/KXTocDir
 		 */
-		for ( path_size = (uint32_t)strlen ( full_path );
+		for ( path_size = (uint32_t)string_measure( full_path, &size );
 		      ( path_size > self->root ) && ( full_path[ path_size - 1 ] == '/' );
 		      -- path_size )
 		{}
@@ -2123,12 +2125,15 @@ static rc_t CC KXTocDirResolvePath (const KXTocDir *self,
     rc_t rc;
     const char * path;
 
+    KOutMsg ("+++++\n%s: absolute %d\n", __func__, absolute);
+
     assert (self);
     assert (resolved);
     assert (path_);
 
     *resolved = '\0';
     rc = XTocMakePath (&path, false, path_, args);
+    KOutMsg ("%s: rc %R new '%s' old '%s'\n", __func__, rc, path, path_);
     if (rc != 0)
     {
         PLOGERR (klogErr,
@@ -2136,24 +2141,41 @@ static rc_t CC KXTocDirResolvePath (const KXTocDir *self,
     }
     else
     {
+        size_t path_sz = string_size(path);
+
         if (absolute)
         {
             XTocEntry * entry;
 
             rc = XTocEntryResolvePath (self->entry, path, false, &entry);
+
             if (rc)
                 PLOGERR (klogErr,
-                         (klogErr, rc, "Error resolvong path based on $(P)", "P=%s", path_));
+                         (klogErr, rc, "Error resolving path based on $(P)", "P=%s", path_));
             else
             {
                 free ((void*)path);
                 path = NULL;
                 rc = XTocEntryMakeFullPath (entry, &path);
+                if (rc == 0)
+                {
+                    size_t k = self->toc->base_path.size;
+                    if (k > 1)
+                    {
+                        memcpy (resolved, self->toc->base_path.addr, k);
+                        resolved += k;
+                        rsize -= k;
+                    }
+                }
             }
         }
         if (rc == 0)
-            string_copy (resolved, rsize, path, string_size (path));
-
+        {
+            if (rsize < (path_sz - 1))
+                rc = RC(rcFS, rcDirectory, rcResolving, rcBuffer, rcInsufficient);
+            else
+                string_copy(resolved, rsize, path, rsize);
+        }
         if (path)
             free ((void*)path);
     }
@@ -2180,11 +2202,11 @@ static rc_t CC KXTocDirResolveAlias (const KXTocDir * self,
 				 const char *path_,
 				 va_list args)
 {
-#if 1
+#if 0
     return -1;
 #else
     rc_t rc;
-    char * path;
+    const char * path;
 
     assert (self);
     assert (resolved);
@@ -2201,13 +2223,13 @@ static rc_t CC KXTocDirResolveAlias (const KXTocDir * self,
     {
         XTocEntry * entry;
 
-        rc = XocEntryResolvePath (self->entry, path, true , &entry);
+        rc = XTocEntryResolvePath (self->entry, path, true , &entry);
         if (rc)
             PLOGERR (klogErr,
-                     (klogErr, rc, "Error resolvong path based on $(P)", "P=%s", path_));
+                     (klogErr, rc, "Error resolving path based on $(P)", "P=%s", path_));
         else
         {
-            free (path);
+            free ((void*)path);
             path = NULL;
             rc = XTocEntryMakeFullPath (entry, &path);
         }
@@ -2220,7 +2242,7 @@ static rc_t CC KXTocDirResolveAlias (const KXTocDir * self,
         string_copy (resolved, rsize, path, string_size (path));
     
     if (path)
-        free (path);
+        free ((void*)path);
 
     return rc;
 #endif
@@ -2402,7 +2424,7 @@ static	rc_t CC KXTocDirVDate		(const KXTocDir *self,
         rc = XTocEntryResolvePath (self->entry, path, false, &entry);
         if (rc)
             PLOGERR (klogErr,
-                     (klogErr, rc, "Error resolveing path from $(P)", "P=%s", path_));
+                     (klogErr, rc, "Error resolving path from $(P)", "P=%s", path_));
         else
             *date = entry->mtime;
         free ((void*)path);
@@ -2739,7 +2761,7 @@ rc_t CC KXTocDirFileLocator		(const KXTocDir *self,
         rc = XTocEntryResolvePath (self->entry, path, false, &entry);
         if (rc)
             PLOGERR (klogErr,
-                     (klogErr, rc, "Error resolveing path from $(P)", "P=%s", path_));
+                     (klogErr, rc, "Error resolving path from $(P)", "P=%s", path_));
         else if (entry->type != xtoce_file)
             rc = RC (rcFS, rcDirectory, rcAccessing, rcPath, rcIncorrect);
         else
@@ -2785,7 +2807,7 @@ rc_t CC KXTocDirFileSize		(const KXTocDir *self,
         rc = XTocEntryResolvePath (self->entry, path, false, &entry);
         if (rc)
             PLOGERR (klogErr,
-                     (klogErr, rc, "Error resolveing path from $(P)", "P=%s", path_));
+                     (klogErr, rc, "Error resolving path from $(P)", "P=%s", path_));
         else if (entry->type != xtoce_file)
             rc = RC (rcFS, rcDirectory, rcAccessing, rcPath, rcIncorrect);
         else
@@ -2831,7 +2853,7 @@ rc_t CC KXTocDirFilePhysicalSize        (const KXTocDir *self,
         rc = XTocEntryResolvePath (self->entry, path, false, &entry);
         if (rc)
             PLOGERR (klogErr,
-                     (klogErr, rc, "Error resolveing path from $(P)", "P=%s", path_));
+                     (klogErr, rc, "Error resolving path from $(P)", "P=%s", path_));
         else if (entry->type != xtoce_file)
             rc = RC (rcFS, rcDirectory, rcAccessing, rcPath, rcIncorrect);
         else
@@ -2886,6 +2908,7 @@ rc_t CC KXTocDirOpenDirRead	(const KXTocDir *self,
 #else
     char * full;
     rc_t rc;
+    size_t size;
 
     assert (self != NULL);
     assert (subp != NULL);
@@ -2896,7 +2919,7 @@ rc_t CC KXTocDirOpenDirRead	(const KXTocDir *self,
     {
 	const KTocEntry *	pnode;
 	KTocEntryType		type;
-	size_t path_size = strlen (full);
+	size_t path_size = string_measure (full, &size);
 
 	/* -----
 	 * get rid of any extra '/' characters at the end of path
@@ -3163,9 +3186,70 @@ rc_t KXTocDirMake (const KXTocDir ** pself, const KDirectory * base,
 }
 
 
-LIB_EXPORT rc_t CC KDirectoryOpenXTocDirRead (const KDirectory * self,
-                                              const KDirectory ** pnew_dir,
-                                              const char * base_path, const KFile * xml)
+static 
+rc_t KDirectoryOpenXTocDirReadInt (const KDirectory * dir,
+                                   const KDirectory ** pnew_dir,
+                                   const KFile * xml,
+                                   const String * spath)
+{
+    XToc * xtoc;
+    rc_t rc;
+
+    rc = XTocMake (&xtoc, spath);
+    if (rc)
+        LOGERR (klogErr, rc, "Error creating toc for xtc directory");
+    else
+    {
+        String sroot;
+        XTocCache * cache;
+
+        StringInitCString (&sroot, ".");
+        rc = XTocMakeXTocCache (xtoc, &cache, &sroot, xtoc->root);
+        if (rc)
+            LOGERR (klogErr, rc, 
+                    "Failed to make cache for root for "
+                    "xtoc directory");
+        else
+        {
+            const KXTocDir * xdir;
+
+            /* now that we have a cache node we should release
+             * the original reference to the xtoc */
+/* ??? */
+            XTocRelease (xtoc);
+
+            xtoc->cache = xtoc->root->cache = cache;
+
+            rc = KXTocDirMake (&xdir, dir, xtoc, cache, xtoc->root,
+                               &sroot);
+            if (rc)
+                LOGERR (klogErr, rc, "Error making xtoc directory");
+            else
+            {
+                DBG_KXTocDir((xdir));
+                cache->dir = xdir;
+
+                rc = XTocParseXml (xtoc->root, xml);
+                if (rc)
+                    LOGERR (klogErr, rc, "Error parsing copycat xml file");
+                else
+                {
+                    *pnew_dir = &xdir->dad;
+                    return 0;
+                }
+                KDirectoryRelease (&xdir->dad);
+            }                
+            XTocCacheRelease (cache);
+        }
+        XTocRelease (xtoc);
+    }
+    return rc;
+}
+
+rc_t CC KDirectoryOpenXTocDirReadDir (const KDirectory * self,
+                                                 const KDirectory ** pnew_dir,
+                                                 const KFile * xml,
+                                                 const String * spath)
 {
     rc_t rc;
     
@@ -3185,115 +3269,198 @@ LIB_EXPORT rc_t CC KDirectoryOpenXTocDirRead (const KDirectory * self,
         LOGERR (klogErr, rc, "self is NULL for opening XToc Directory");
         return rc;
     }
-    if (xml == NULL)
+    else if (xml == NULL)
     {
         rc = RC (rcFS, rcDirectory, rcOpening, rcParam, rcNull);
         LOGERR (klogErr, rc, "xml parameter is NULL for opening XToc Directory");
         return rc;
     }
-    if (base_path == NULL)
+    else if (spath == NULL)
     {
         rc = RC (rcFS, rcDirectory, rcOpening, rcParam, rcNull);
         LOGERR (klogErr, rc, "base path parameter is NULL for opening XToc Directory");
         return rc;
     }
-
-    /* okay none of our parameters are NULL; they could still be bad but let's start */
+    else
     {
-        size_t base_path_size;  /* utf-8 bytes (not characters) in the passed in base path */      
-        char * slash;           /* find the last slash in the given path */
-        char * base_path_dir;   /* path leading up to the file in the path */
-        bool free_base_path_dir = true;
-
-        base_path_size = string_size (base_path);
-        slash = string_rchr (base_path, base_path_size, '/');
-
-        if (slash)
-        {
-            base_path_dir = malloc ((slash - base_path) + 1);
-            if (base_path_dir == NULL)
-            {
-                rc = RC (rcFS, rcDirectory, rcConstructing, rcMemory, rcExhausted);
-                LOGERR (klogErr, rc, "Error allocating memory to build xtoc directory");
-                return rc;
-            }
-            string_copy (base_path_dir, ((slash - base_path) + 1), 
-                         base_path, (slash - base_path));
-        }
-        else
-        {
-            free_base_path_dir = false;
-            base_path_dir = "/";
-        }
-        {
-            const KDirectory * base;
-
-            rc = KDirectoryOpenDirRead (self, &base, true, base_path_dir);
-            if (rc)
-                LOGERR (klogErr, rc, "Error opening directory for xtoc directory");
-            else
-            {
-                String sbase;
-                XToc * toc;
-
-                StringInitCString (&sbase, base_path_dir);
-
-                rc = XTocMake (&toc, &sbase);
-                if (rc)
-                    LOGERR (klogErr, rc, "Error creating toc for xtc directory");
-                else
-                {
-                    XTocCache * cache;
-                    String sroot;
-
-                    StringInitCString (&sroot, ".");
-
-                    rc = XTocMakeXTocCache (toc, &cache, &sroot, toc->root);
-                    if (rc)
-                        LOGERR (klogErr, rc, 
-                                "Failed to make cache for root for "
-                                "xtoc directory");
-                    else
-                    {
-                        const KXTocDir * xdir;
-
-                        /* now that we have a cache node we should release
-                         * the original reference to the xtoc */
-                        XTocRelease (toc);
-
-                        toc->cache = toc->root->cache = cache;
-
-                        rc = KXTocDirMake (&xdir, base, toc, cache, toc->root,
-                                           &sroot);
-                        /* done with our reference to base */
-                        if (rc)
-                            LOGERR (klogErr, rc, "Error making xtoc directory");
-                        else
-                        {
-                            DBG_KXTocDir((xdir));
-                            cache->dir = xdir;
-
-                            rc = XTocParseXml (toc->root, xml);
-                            if (rc)
-                                LOGERR (klogErr, rc, "Error parsing copycat xml file");
-                            else
-                            {
-                                *pnew_dir = &xdir->dad;
-                                if (free_base_path_dir)
-                                    free (base_path_dir);
-                                return 0;
-                            }
-                            KDirectoryRelease (&xdir->dad);
-                        }
-                    }
-                }
-                KDirectoryRelease (base);
-            }
-            if (free_base_path_dir)
-                free (base_path_dir);
-        } 
-   }
+        rc = KDirectoryOpenXTocDirReadInt (self, pnew_dir, xml, spath);
+    }
     return rc;
 }
 
+
+rc_t CC KDirectoryVOpenXTocDirRead (const KDirectory * self,
+                                               const KDirectory ** pnew_dir,
+                                               bool chroot,
+                                               const KFile * xml,
+                                               const char * _path,
+                                               va_list args )
+{
+    rc_t rc;
+    
+    /* loosely validate parameters */
+    if (pnew_dir == NULL)
+    {
+        rc = RC (rcFS, rcDirectory, rcOpening, rcParam, rcNull);
+        LOGERR (klogErr, rc,
+                "new directory parameter is NULL for opening XToc Directory");
+        return rc;
+    }
+    *pnew_dir = NULL;
+
+    if (self == NULL)
+    {
+        rc = RC (rcFS, rcDirectory, rcOpening, rcSelf, rcNull);
+        LOGERR (klogErr, rc, "self is NULL for opening XToc Directory");
+        return rc;
+    }
+    else if (xml == NULL)
+    {
+        rc = RC (rcFS, rcDirectory, rcOpening, rcParam, rcNull);
+        LOGERR (klogErr, rc, "xml parameter is NULL for opening XToc Directory");
+        return rc;
+    }
+    else if (_path == NULL)
+    {
+        rc = RC (rcFS, rcDirectory, rcOpening, rcParam, rcNull);
+        LOGERR (klogErr, rc, "base path parameter is NULL for opening XToc Directory");
+        return rc;
+    }
+    else
+    /* okay none of our parameters are NULL; they could still be bad but let's start */
+    {
+        String spath;
+        const KDirectory * bdir;
+        KPathType type;
+        char path [8192];
+
+        rc = KDirectoryVResolvePath (self, true, path, sizeof path, _path, args);
+        if (rc)
+            return rc;
+
+        type = KDirectoryPathType (self, path);
+        switch (type & ~kptAlias)
+        {
+        case kptNotFound:
+            rc = RC (rcFS, rcDirectory, rcOpening, rcParam, rcNotFound);
+            LOGERR (klogErr, rc, "base path parameter is not found when opening XToc Directory");
+            return rc;
+
+        case kptBadPath:
+            rc = RC (rcFS, rcDirectory, rcOpening, rcParam, rcInvalid);
+            LOGERR (klogErr, rc, "base path parameter is a bad path when opening XToc Directory");
+            return rc;
+
+        case kptFile:
+        case kptCharDev:
+        case kptBlockDev:
+        case kptFIFO:
+        case kptZombieFile:
+        case kptDataset:
+        case kptDatatype:
+            KOutMsg ("%s: type '%u' path '%s'\n",type,path);
+            rc = RC (rcFS, rcDirectory, rcOpening, rcParam, rcWrongType);
+            LOGERR (klogErr, rc, "base path parameter is an unusable type when opening XToc Directory");
+            return rc;
+
+        default:
+            rc = RC (rcFS, rcDirectory, rcOpening, rcParam, rcCorrupt);
+            LOGERR (klogErr, rc, "base path parameter is not a known type when opening XToc Directory");
+            return rc;
+
+        case kptDir:
+            break;
+        }
+
+        rc = KDirectoryVOpenDirRead (self, &bdir, true, path, args);
+        if (rc)
+        {
+            LOGERR (klogErr, rc, "failed to open base directory for XToc directory");
+            return rc;
+        }
+        else
+        {
+            static const char absroot[] = "/";
+
+            if (chroot)
+                StringInitCString (&spath, absroot);
+            else
+                StringInitCString (&spath, path);
+
+            rc = KDirectoryOpenXTocDirReadInt (bdir, pnew_dir, xml, &spath);
+            if (rc == 0)
+                return 0;
+
+            KDirectoryRelease (bdir);
+        }
+    }
+    return rc;
+}
+
+
+rc_t CC KDirectoryOpenXTocDirRead (const KDirectory * self,
+                                              const KDirectory ** pnew_dir,
+                                              bool chroot,
+                                              const KFile * xml,
+                                              const char * path, ... )
+{
+    rc_t rc;
+    va_list args;
+
+    va_start (args, path);
+    rc = KDirectoryVOpenXTocDirRead (self, pnew_dir, chroot, xml, path, args);
+    va_end (args);
+
+    return rc;
+}
+
+
+struct VFSManager;
+
+rc_t CC VFSManagerOpenXTocDirRead (const struct VFSManager * self,
+                                              const struct KDirectory ** pnew_dir,
+                                              const struct KFile * xml,
+                                              const struct VPath * path)
+{
+    rc_t rc = 0;
+
+    /* needs to be replaced with proper checking and rc returns */
+    assert (pnew_dir);
+
+    *pnew_dir = NULL;
+
+    assert (self);
+    assert (xml);
+    assert (path);
+
+    if (rc == 0)
+    {
+        const KDirectory * dir;
+
+        rc = VFSManagerOpenDirectoryRead (self, &dir, path);
+        if (rc == 0)
+        {
+            char pbuff [8192];
+            size_t z;
+
+            rc = VPathReadPath (path, pbuff, sizeof pbuff, &z);
+            if (rc == 0)
+            {
+                String string;
+
+                StringInit (&string, pbuff, z, string_len (pbuff, z));
+
+                rc = KDirectoryOpenXTocDirReadDir (dir, pnew_dir, xml, &string);
+                if (rc == 0)
+                    return 0;
+            }
+
+            KDirectoryRelease (dir);
+        }
+    }
+    return rc;
+}
+
+
+
 /* end of file */
