@@ -47,8 +47,8 @@
 #include <klib/namelist.h>
 #include <klib/data-buffer.h>
 #include <klib/printf.h>
+#include <klib/out.h>
 #include <klib/rc.h>
-#include <fmtdef.h>
 #include <os-native.h>      /* because of snprintf on windows */
 #include <sysalloc.h>
 
@@ -181,11 +181,11 @@ void CC KSymbolList ( BSTNode *item, void *data )
     const KSymbol *self = ( const KSymbol* ) item;
     const char *indent = ( const char* ) data;
 
-    printf ( "%s%.*s = 0x%" LUSZ
+    OUTMSG (( "%s%.*s = %#p"
              , indent
              , ( int ) self -> name . size, self -> name . addr
-             , ( size_t ) self
-        );
+             , self
+        ));
 
     if ( self -> type != eNamespace )
         putchar ( '\n' );
@@ -202,10 +202,10 @@ void CC KSymbolList ( BSTNode *item, void *data )
 static
 void CC BSTreeListSymbols ( const BSTree *self, const char *indent )
 {
-    printf ( "%sscope = 0x%" LUSZ "\n"
+    OUTMSG (( "%sscope = %#p\n"
              , indent
-             , ( size_t ) self
-        );
+             , self
+        ));
 
     if ( indent [ -1 ] != '.' )
         -- indent;
@@ -570,7 +570,7 @@ LIB_EXPORT rc_t CC VSchemaRelease ( const VSchema *self )
         {
         case krefWhack:
             return VSchemaWhack ( ( VSchema* ) self );
-        case krefLimit:
+        case krefNegative:
             return RC ( rcVDB, rcMgr, rcReleasing, rcRange, rcExcessive );
         }
     }
@@ -601,7 +601,7 @@ rc_t VSchemaSever ( const VSchema *self )
         {
         case krefWhack:
             return VSchemaWhack ( ( VSchema* ) self );
-        case krefLimit:
+        case krefNegative:
             return RC ( rcVDB, rcMgr, rcReleasing, rcRange, rcExcessive );
         }
     }
@@ -741,7 +741,8 @@ rc_t CC VSchemaParseTextInt ( VSchema *self,
     KTokenText tt;
     KTokenSource src;
     String str, path;
-
+    rc_t rc;
+    
     if ( name == NULL || name [ 0 ] == 0 )
         CONST_STRING ( & path, "<unnamed>" );
     else
@@ -751,7 +752,14 @@ rc_t CC VSchemaParseTextInt ( VSchema *self,
     KTokenTextInit ( & tt, & str, & path );
     KTokenSourceInit ( & src, & tt );
 
-    return schema ( & src, self );
+    rc = schema ( & src, self );
+    
+    if (rc == 0)
+        PARSE_DEBUG( ("Parsed schema from %s\n", name) );
+    else
+        PARSE_DEBUG( ("Failed to parse schema from %s\n", name) );
+    
+    return rc;
 }
 
 LIB_EXPORT rc_t CC VSchemaParseText ( VSchema *self, const char *name,
