@@ -46,7 +46,6 @@
 
 #include <kapp/main.h>
 #include <kapp/args.h>
-#include <fmtdef.h>
 
 
 #include <stdio.h>
@@ -93,7 +92,7 @@ static const char * directory_usage[] =
 static const char * force_usage[] =
 { "(no parameter) this will cause the extract or",
   "create to over-write existing files unless",
-  "they are write-protected.  without this",
+  "they are write-protected.  Without this",
   "option the program will fail if the archive",
   "already exists for a create or the target",
   "directory exists for an extract", NULL };
@@ -164,10 +163,12 @@ rc_t CC Usage (const Args * args)
     HelpOptionLine (ALIAS_TEST, OPTION_TEST, archive, test_usage);
     OUTMSG (("\n"
              "Archive:\n"
-             "  Path to a file that will/does hold the archive of other files\n"
+             "  Path to a file that will/does hold the archive of other files.\n"
+             "  This can be a full or relative path.\n"
              "\n"
              "Directory:\n"
-	     "    Required for create or extract command, ignored for test command\n"
+	     "  Required for create or extract command, ignored for test command.\n"
+             "  This can be a full or relative path.\n"
              "\n"
              "Filters:\n"
 	     "  When present these act as include filters.\n"
@@ -183,6 +184,37 @@ rc_t CC Usage (const Args * args)
 
     HelpOptionsStandard ();
 
+    OUTMSG (("\n"
+             "Use examples:"
+             "\n"
+             "  To create an archive named 'example.sra' that contains the same\n"
+             "  contents as a subdirectory 'example' of the current directory\n"
+             "\n"
+             "  $ %s --%s example.sra --%s example\n",
+             progname, OPTION_CREATE, OPTION_DIRECTORY));
+
+    OUTMSG (("\n"
+             "  To replace an existing archive named 'example.sra' with another that contains\n"
+             "  the same contents as a subdirectory 'example' of the current directory\n"
+             "\n"
+             "  $ %s -%s -%s example.sra -%s example\n",
+             progname, ALIAS_FORCE, ALIAS_CREATE, ALIAS_DIRECTORY));
+
+    OUTMSG (("\n"
+             "  To examine in detail the contents of an archive named 'example.sra'\n"
+             "\n"
+             "  $ %s --%s --%s example.sra\n",
+             progname, OPTION_LONGLIST, OPTION_TEST));
+
+    OUTMSG (("\n"
+             "  To extract the files from an archive named 'example.sra' into\n"
+             "  a subdirectory 'example' of the current directory.\n"
+             "  NOTE: all extracted files will be read only.\n"
+             "\n"
+             "  $ %s --%s example.sra --%s example\n",
+             progname, OPTION_EXTRACT, OPTION_DIRECTORY));
+
+
     HelpVersion (fullpath, KAppVersion());
 
     return rc;
@@ -327,6 +359,54 @@ static KSRAFileAlignment get_alignment (const char * str)
     }
     return sraAlignInvalid;
 }
+
+static
+int CC sort_cmp (const void ** l, const void ** r, void * data)
+{
+    KDirectory * d;
+    uint64_t lz, rz;
+    rc_t rc;
+
+    d = data;
+/*     lz = l; */
+/*     rz = r; */
+
+    rc = KDirectoryFileSize (data, &lz, *l);
+    if (rc == 0)
+    {
+        rc = KDirectoryFileSize (data, &rz, *r);
+        if (rc == 0)
+        {
+            int64_t zdiff;
+
+            zdiff = lz - rz;
+            if (zdiff != 0)
+                return (zdiff < 0) ? -1 : 1;
+            else
+            {
+                size_t lsz = string_size (*l);
+                size_t rsz = string_size (*r);
+
+                return string_cmp (*l, lsz, *r, rsz, lsz+1);
+            }
+        }
+    }
+    return 0; /* dunno just leave it... */
+}
+
+
+static
+rc_t CC sort_size_then_rel_path (const KDirectory * d, struct Vector * v)
+{
+    VectorReorder (v, sort_cmp, (void*)d);
+
+    return 0;
+}
+
+
+
+
+
 static
 rc_t open_dir_as_archive (const char * path, const KFile ** file)
 {
@@ -356,11 +436,13 @@ rc_t open_dir_as_archive (const char * path, const KFile ** file)
         PLOGERR (klogFatal, (klogFatal, rc, "Parameter [$(P)] must be a directory", PLOG_S(P), path));
         return rc;
     case kptDir:
+/*         KOutMsg ("%s: opening dir\n",__func__); */
         rc = KDirectoryVOpenDirRead (kdir, &d, false, path, NULL);
     }
     if (rc == 0)
     {
-        rc = KDirectoryOpenTocFileRead ( d, file, alignment, pnamesFilter, NULL, NULL );
+/*         KOutMsg ("%s: dir to archive\n",__func__); */
+        rc = KDirectoryOpenTocFileRead (d, file, alignment, pnamesFilter, NULL, sort_size_then_rel_path );
         KDirectoryRelease (d);
     }
     return rc;
@@ -466,6 +548,14 @@ rc_t open_out_file (const char * path, KFile ** fout)
     }
     return rc;
 }
+
+
+static
+void remove_out_file (const char * path)
+{
+    KDirectoryRemove (kdir, true, path);
+}
+
 static
 rc_t copy_file (const KFile * fin, KFile *fout)
 {
@@ -612,9 +702,12 @@ rc_t step_through_dir (const KDirectory * dir, const char * path,
                     char * new_path;
                     size_t namelen;
                     size_t new_pathlen;
+                    size_t rsize;
+
                     namelen = strlen (name);
                     new_pathlen = pathlen + 1 + namelen;
-                    new_path = malloc (new_pathlen + 1);
+                    rsize = new_pathlen + 1;
+                    new_path = malloc (rsize);
                     if (new_path != NULL)
                     {
                         char * recur_path;
@@ -627,15 +720,14 @@ rc_t step_through_dir (const KDirectory * dir, const char * path,
                         {
                             memcpy (new_path, path, pathlen);
                             new_path[pathlen] = '/';
-                            memcpy (new_path + pathlen + 1, name, namelen);
-                            new_path[pathlen+1+namelen] = '\0';
+                            memcpy (&new_path[pathlen+1], name, namelen);
+                            new_path[new_pathlen] = '\0';
                         }
-                        recur_path = malloc (pathlen + 1 + namelen + 1);
+                        recur_path = malloc (rsize);
                         if (recur_path != NULL)
                         {
                             rc = KDirectoryVResolvePath (dir, false, recur_path,
-                                                         pathlen + 1 + namelen + 1,
-                                                         new_path, NULL);
+                                                         rsize, new_path, NULL);
 
                             if (rc == 0)
                             {
@@ -761,6 +853,12 @@ rc_t	run_kar_create(const char * archive, const char * directory)
         }
         KFileRelease (fout);
         free (directorystr);
+
+        if (rc)
+        {
+            remove_out_file (archive);
+        }
+
     }
     return rc;
 }
