@@ -32,6 +32,15 @@
  * for the "big loop"
  */
 
+
+/* The original intention of this tool was to support updating nodes.
+   This was later removed with the thought that this functionality would
+   be better implemented by another tool.
+
+   Going to temporarily add it in...
+*/
+#define ALLOW_UPDATE 1
+
 #include "kdbmeta.vers.h"
 
 #include <sra/srapath.h>
@@ -61,25 +70,24 @@
 #include <assert.h>
 #include <ctype.h>
 
+/* use cpp define CONST */
+#undef CONST
+#if ALLOW_UPDATE
+#define CONST
+#else
+#define CONST const
+#endif
 
-enum
+typedef struct KDBMetaParms KDBMetaParms;
+struct KDBMetaParms
 {
-    kptMetadata = kptColumn + 1,
-    kptPrereleaseTbl
-};
-
-
-typedef struct KDBMetaConst KDBMetaConst;
-struct KDBMetaConst
-{
-    const KDBManager *mgr;
-    const KMetadata *md;
+    CONST KDBManager *mgr;
+    CONST KMetadata *md;
     const char *targ;
     const Vector *q;
     rc_t rc;
 };
 
-
 /* select reporting
  */
 static bool xml_ish = true;
@@ -87,6 +95,8 @@ static int indent_lvl;
 static int tabsz = 2;
 static const char *spaces = "                                ";
 static bool as_unsigned = false;
+static const char *table_arg = NULL;
+static bool read_only_arg = true;
 
 static
 void indent ( void )
@@ -107,13 +117,25 @@ void indent ( void )
 static
 void node_open ( const char *path, size_t plen )
 {
+    if ( plen && path[plen - 1] == '/' ) {
+    /* do not use trailing slash */
+        --plen;
+    }
     if ( xml_ish )
     {
+        int size = plen;
         /* find leaf name of node */
         const char *slash = string_rchr ( path, plen, '/' );
+        if ( slash ) {
+            /* make sure not to print trailing slash */
+            size = plen - (slash - path) - 1;
+            if ( size < 0 ) {
+                size = 0;
+            }
+        }
 
         indent ();
-        OUTMSG (( "<%s", ( slash != NULL ) ? slash + 1 : path ));
+        OUTMSG (( "<%.*s", size, ( slash != NULL ) ? slash + 1 : path ));
     }
 }
 
@@ -121,16 +143,26 @@ static
 void node_close ( const char *path, size_t plen,
     size_t vsize, uint32_t num_children, bool close_indent )
 {
+    if ( plen && path[plen - 1] == '/' ) {
+        --plen;
+    }
     if ( xml_ish )
     {
         if ( vsize == 0 && num_children == 0 )
             OUTMSG (( "/>\n" ));
         else
         {
+            int size = plen;
             const char *slash = string_rchr ( path, plen, '/' );
+            if ( slash ) {
+                size = plen - (slash - path) - 1;
+                if ( size < 0 ) {
+                    size = 0;
+                }
+            }
             if ( close_indent )
                 indent ();
-            OUTMSG (( "</%s>\n", ( slash != NULL ) ? slash + 1 : path ));
+            OUTMSG (( "</%.*s>\n", size, ( slash != NULL ) ? slash + 1 : path ));
         }
     }
 }
@@ -328,6 +360,9 @@ rc_t md_select_expr ( const KMDataNode *node, char *path, size_t psize, int plen
 
             /* close it off */
             node_close ( path, plen, 0, 0, false );
+
+            /* Exit here if an attribute was requested in query */
+            return 0;
         }
         else
         {
@@ -476,68 +511,182 @@ rc_t md_select_expr ( const KMDataNode *node, char *path, size_t psize, int plen
     return 0;
 }
 
+#if ALLOW_UPDATE
+static
+rc_t md_update_expr ( KMDataNode *node, const char *path, const char *attr, const char *expr )
+{
+    rc_t rc;
+
+    /* according to documentation, "expr" is allowed to be text
+       or text with escaped hex sequences. examine for escaped hex */
+    size_t len = string_size ( expr );
+    char *buff = malloc ( len + 1 );
+    if ( buff == NULL )
+        rc = RC ( rcExe, rcMetadata, rcUpdating, rcMemory, rcExhausted );
+    else
+    {
+        size_t i, j;
+        for ( i = j = 0; i < len; ++ i, ++ j )
+        {
+            if ( ( buff [ j ] = expr [ i ] ) == '\\' )
+            {
+                /* we know "expr" is NUL-terminated, so this is safe */
+                if ( tolower ( expr [ i + 1 ] ) == 'x' &&
+                     isxdigit ( expr [ i + 2 ] ) &&
+                     isxdigit ( expr [ i + 3 ] ) )
+                {
+                    int msn = toupper ( expr [ i + 2 ] ) - '0';
+                    int lsn = toupper ( expr [ i + 3 ] ) - '0';
+                    if ( msn >= 10 )
+                        msn += '0' - 'A' + 10;
+                    if ( lsn >= 10 )
+                        lsn += '0' - 'A' + 10;
+                    buff [ j ] = ( char ) ( ( msn << 4 ) | lsn );
+                    i += 3;
+                }
+            }
+        }
+
+        if ( attr != NULL )
+        {
+            /* set attribute value */
+            buff [ j ] = 0;
+            rc = KMDataNodeWriteAttr ( node, attr, buff );
+        }
+        else
+        {
+            /* now set the value of the node */
+            rc = KMDataNodeWrite ( node, buff, j );
+        }
+        free ( buff );
+    }
+
+    return rc;
+}
+#endif
+
 static
 bool CC md_select ( void *item, void *data )
 {
     bool fail = true;
-    KDBMetaConst *pb = data;
+    KDBMetaParms *pb = data;
 
-    const KMDataNode *node;
-    pb -> rc = KMetadataOpenNodeRead ( pb -> md, & node, NULL );
+    CONST KMDataNode *node;
+#if ALLOW_UPDATE
+    bool read_only = true;
+    if ( ! read_only_arg ) {
+        pb -> rc = KMetadataOpenNodeUpdate ( pb -> md, & node, NULL );
+        if ( pb -> rc == 0 )
+            read_only = false;
+    }
+#endif
+    if ( read_only )
+        pb -> rc = KMetadataOpenNodeRead ( pb -> md, ( const KMDataNode** ) & node, NULL );
     if ( pb -> rc != 0 )
         PLOGERR ( klogErr,  (klogErr, pb -> rc, "failed to open root node for '$(path)'", "path=%s", pb -> targ ));
     else
     {
-        const char *expr = ( const void* ) item;
-        const char *attr = strchr ( expr, '@' );
+        bool wildcard;
+        char *expr, *attr, path [ 4096 ];
+        size_t len = string_copy_measure ( path, sizeof path, item );
 
-        size_t len;
-        char path [ 4096 ];
-
-        pb -> rc = attr == NULL ?
-            string_printf ( path, sizeof path, & len, "%s", expr ):
-            string_printf ( path, sizeof path, & len, "%.*s", ( uint32_t ) ( attr ++ - expr ), expr );
-        if ( pb -> rc != 0 )
-            PLOGERR ( klogErr,  (klogErr, pb -> rc, "failed to evaluate query '$(expr)'", "expr=%s", expr ));
-        else
+        /* detect assignment */
+        expr = string_rchr ( path, len, '=' );
+        if ( expr != NULL )
         {
-            bool wildcard = false;
-            if ( len >= 1 && path [ len - 1 ] == '*' )
+            len = expr - path;
+            * expr ++ = 0;
+        }
+        attr = string_rchr ( path, len, '@' );
+        if ( attr != NULL )
+        {
+            len = attr - path;
+            * attr ++ = 0;
+        }
+
+        if ( expr != NULL )
+        {
+#if ALLOW_UPDATE
+            if ( read_only )
             {
-                if ( len == 1 )
-                {
-                    path [ len = 0 ] = 0;
-                    wildcard = true;
-                }
-                else if ( len >= 2 && path [ len - 2 ] == '/' )
-                {
-                    path [ len -= 2 ] = 0;
-                    wildcard = true;
-                }
+                PLOGMSG ( klogWarn, ( klogWarn, "node update expressions are not supported - "
+                                      "'$(path)' is read-only - "
+                                      "'$(expr)' treated as select."
+                                      , "path=%s,expr=%s", pb -> targ, item ) );
+                expr = NULL;
+            }
+#else
+            PLOGMSG ( klogWarn, ( klogWarn, "node update expressions are not supported - "
+                                  "'$(expr)' treated as select.", "expr=%s", item ) );
+            expr = NULL;
+#endif
+        }
+
+        wildcard = false;
+        if ( len >= 1 && path [ len - 1 ] == '*' )
+        {
+            if ( len == 1 )
+            {
+                path [ len = 0 ] = 0;
+                wildcard = true;
+            }
+            else if ( len >= 2 && path [ len - 2 ] == '/' )
+            {
+                path [ len -= 2 ] = 0;
+                wildcard = true;
+            }
+        }
+
+#if ALLOW_UPDATE
+        if ( expr != NULL )
+        {
+            KMDataNode *root = node;
+
+            if ( wildcard || len == 0 )
+            {
+                pb -> rc = RC ( rcExe, rcMetadata, rcUpdating, rcExpression, rcIncorrect );
+                PLOGERR ( klogErr, ( klogErr, pb -> rc, "node updates require explicit paths - "
+                                     "'$(expr)' cannot be evaluated", "expr=%s", item ) );
+                return true;
             }
 
-            if ( len == 0 )
-                pb -> rc = md_select_expr ( node, path, sizeof path, len, attr, wildcard );
+            pb -> rc = KMDataNodeOpenNodeUpdate ( root, & node, path );
+            KMDataNodeRelease ( root );
+
+            if ( pb -> rc != 0 )
+            {
+                PLOGERR ( klogErr,  (klogErr, pb -> rc, "failed to open node '$(node) for '$(path)'",
+                                     "node=%s,path=%s", path, pb -> targ ));
+            }
             else
             {
-                const KMDataNode *root = node;
-                pb -> rc = KMDataNodeOpenNodeRead ( root, & node, path );
-                KMDataNodeRelease ( root );
-
-                if ( pb -> rc != 0 )
-                {
-                    PLOGERR ( klogErr,  (klogErr, pb -> rc, "failed to open node '$(node) for '$(path)'",
-                                         "node=%s,path=%s", path, pb -> targ ));
-                }
-                else
-                {
-                    pb -> rc = md_select_expr ( node, path, sizeof path, len, attr, wildcard );
-                }
+                pb -> rc = md_update_expr ( node, path, attr, expr );
             }
-
-            if ( pb -> rc == 0 )
-                fail = false;
         }
+        else
+#endif
+
+        if ( len == 0 )
+            pb -> rc = md_select_expr ( node, path, sizeof path, len, attr, wildcard );
+        else
+        {
+            const KMDataNode *root = node;
+            pb -> rc = KMDataNodeOpenNodeRead ( root, ( const KMDataNode** ) & node, path );
+            KMDataNodeRelease ( root );
+
+            if ( pb -> rc != 0 )
+            {
+                PLOGERR ( klogErr,  (klogErr, pb -> rc, "failed to open node '$(node) for '$(path)'",
+                                     "node=%s,path=%s", path, pb -> targ ));
+            }
+            else
+            {
+                pb -> rc = md_select_expr ( node, path, sizeof path, len, attr, wildcard );
+            }
+        }
+
+        if ( pb -> rc == 0 )
+            fail = false;
 
         KMDataNodeRelease ( node );
     }
@@ -546,17 +695,38 @@ bool CC md_select ( void *item, void *data )
 }
 
 static
-rc_t col_select ( KDBMetaConst * pb)
+rc_t col_select ( KDBMetaParms * pb)
 {
-    const KColumn *col;
+    CONST KColumn *col;
     rc_t rc;
 
-    rc = KDBManagerOpenColumnRead ( pb -> mgr, & col, pb->targ );
+    bool read_only = true;
+
+#if ALLOW_UPDATE
+    if ( ! read_only_arg ) {
+        read_only = false;
+
+        rc = KDBManagerOpenColumnUpdate ( pb -> mgr, & col, pb->targ );
+        if ( rc != 0 )
+            read_only = true;
+    }
+#endif
+    if ( read_only )
+        rc = KDBManagerOpenColumnRead ( pb -> mgr, ( const KColumn** ) & col, pb->targ );
     if ( rc != 0 )
         PLOGERR ( klogErr,  (klogErr, rc, "failed to open column '$(col)'", "col=%s", pb->targ ));
     else
     {
-        rc = KColumnOpenMetadataRead ( col, & pb -> md );
+#if ALLOW_UPDATE
+        if ( ! read_only_arg ) {
+            read_only = false;
+            rc = KColumnOpenMetadataUpdate ( col, & pb -> md );
+            if ( rc != 0 )
+                read_only = true;
+        }
+#endif
+        if ( read_only )
+            rc = KColumnOpenMetadataRead ( col, ( const KMetadata** ) & pb -> md );
         if ( rc != 0 )
             PLOGERR ( klogErr,  (klogErr, rc, "failed to open metadata for column '$(col)'", "col=%s", pb->targ ));
         else
@@ -577,17 +747,39 @@ rc_t col_select ( KDBMetaConst * pb)
 }
 
 static
-rc_t tbl_select ( KDBMetaConst * pb)
+rc_t tbl_select ( KDBMetaParms * pb)
 {
-    const KTable *tbl;
+    CONST KTable *tbl;
     rc_t rc;
 
-    rc = KDBManagerOpenTableRead ( pb -> mgr, & tbl, pb->targ );
+    bool read_only = true;
+
+#if ALLOW_UPDATE
+    if ( ! read_only_arg ) {
+        read_only = false;
+
+        rc = KDBManagerOpenTableUpdate ( pb -> mgr, & tbl, pb->targ );
+        if ( rc != 0 )
+            read_only = true;
+    }
+#endif
+    if ( read_only )
+        rc = KDBManagerOpenTableRead ( pb -> mgr, ( const KTable** ) & tbl, pb->targ );
     if ( rc != 0 )
         PLOGERR ( klogErr,  (klogErr, rc, "failed to open table '$(tbl)'", "tbl=%s", pb->targ ));
     else
     {
-        rc = KTableOpenMetadataRead ( tbl, & pb -> md );
+        read_only = true;
+#if ALLOW_UPDATE
+        if ( ! read_only_arg ) {
+            read_only = false;
+            rc = KTableOpenMetadataUpdate ( tbl, & pb -> md );
+            if ( rc != 0 )
+                read_only = true;
+        }
+#endif
+        if ( read_only )
+            rc = KTableOpenMetadataRead ( tbl, ( const KMetadata** ) & pb -> md );
         if ( rc != 0 )
             PLOGERR ( klogErr,  (klogErr, rc, "failed to open metadata for table '$(tbl)'", "tbl=%s", pb->targ ));
         else
@@ -608,29 +800,94 @@ rc_t tbl_select ( KDBMetaConst * pb)
 }
 
 static
-rc_t db_select (KDBMetaConst * pb)
+rc_t db_select (KDBMetaParms * pb)
 {
-    const KDatabase *db;
+    CONST KDatabase *db;
     rc_t rc;
 
-    rc = KDBManagerOpenDBRead ( pb -> mgr, & db, pb->targ );
-    if ( pb -> rc != 0 )
-        PLOGERR ( klogErr,  (klogErr, rc, "failed to open db '$(db)'", "db=%s", pb->targ ));
-    else
-    {
-        rc = KDatabaseOpenMetadataRead ( db, & pb -> md );
+    bool read_only = true;
+
+#if ALLOW_UPDATE
+    if ( ! read_only_arg ) {
+        read_only = false;
+
+        rc = KDBManagerOpenDBUpdate ( pb -> mgr, & db, pb->targ );
         if ( rc != 0 )
-            PLOGERR ( klogErr,  (klogErr, rc, "failed to open metadata for db '$(db)'", "db=%s", pb->targ ));
-        else
-        {
-            bool fail;
-
-            fail = VectorDoUntil ( pb -> q, false, md_select, pb );
-            if(fail)
-                rc = pb->rc;
-            KMetadataRelease ( pb -> md ), pb -> md = NULL;
+            read_only = true;
+    }
+#endif
+    if ( read_only )
+        rc = KDBManagerOpenDBRead ( pb -> mgr, ( const KDatabase** ) & db, pb->targ );
+    if ( rc != 0 ) {
+        PLOGERR ( klogErr,  (klogErr, rc, "failed to open db '$(db)'",
+            "db=%s", pb->targ ));
+    }
+    else {
+        CONST KTable* tbl = NULL;
+        if (table_arg) {
+            read_only = true;
+#if ALLOW_UPDATE
+            if ( ! read_only_arg ) {
+                read_only = false;
+                rc = KDatabaseOpenTableUpdate ( db, &tbl, table_arg );
+                if ( rc != 0 )
+                    read_only = true;
+            }
+#endif
+            if ( read_only )
+                rc = KDatabaseOpenTableRead ( db, ( const KTable** ) &tbl, table_arg );
+            if ( rc != 0 ) {
+                PLOGERR ( klogErr,  (klogErr, rc,
+                    "failed to open table '$(table)'", "table=%s", table_arg ));
+            }
         }
+        if ( rc == 0) {
+            if (tbl) {
+                read_only = true;
+#if ALLOW_UPDATE
+                if ( ! read_only_arg ) {
+                    read_only = false;
+                    rc = KTableOpenMetadataUpdate ( tbl, & pb -> md );
+                    if ( rc != 0 )
+                        read_only = true;
+                }
+#endif
+                if ( read_only )
+                    rc = KTableOpenMetadataRead ( tbl, ( const KMetadata** ) & pb -> md );
+                if ( rc != 0 ) {
+                    PLOGERR ( klogErr,  (klogErr, rc,
+                        "failed to open metadata for table '$(table)'",
+                        "table=%s", table_arg ));
+                }
+            }
+            else {
+                read_only = true;
+#if ALLOW_UPDATE
+                if ( ! read_only_arg ) {
+                    read_only = false;
+                    rc = KDatabaseOpenMetadataUpdate ( db, & pb -> md );
+                    if ( rc != 0 )
+                        read_only = true;
+                }
+#endif
+                if ( read_only )
+                    rc = KDatabaseOpenMetadataRead ( db, ( const KMetadata** ) & pb -> md );
+                if ( rc != 0 ) {
+                    PLOGERR ( klogErr,  (klogErr, rc,
+                        "failed to open metadata for db '$(db)'",
+                        "db=%s", pb->targ ));
+                }
+            }
+            if ( rc == 0 ) {
+                bool fail;
 
+                fail = VectorDoUntil ( pb -> q, false, md_select, pb );
+                if(fail)
+                    rc = pb->rc;
+                KMetadataRelease ( pb -> md ), pb -> md = NULL;
+            }
+        }
+        KTableRelease ( tbl );
         KDatabaseRelease ( db );
     }
 
@@ -638,9 +895,9 @@ rc_t db_select (KDBMetaConst * pb)
 }
 
 static
-rc_t tool_select ( const KDBManager * mgr, uint32_t type, const char * targ, const Vector * q)
+rc_t tool_select ( CONST KDBManager * mgr, uint32_t type, const char * targ, const Vector * q)
 {
-    KDBMetaConst pb;
+    KDBMetaParms pb;
     rc_t rc;
 
     pb.mgr = mgr;
@@ -694,8 +951,11 @@ rc_t CC UsageSummary (const char * progname)
                     "  %s [Options] <target> [<query> ...]\n"
                     "\n"
                     "Summary:\n"
-                    "  Display the contents of one or more metadata stores.\n",
-                    progname);
+                    "  Display the contents of one or more metadata stores.\n"
+#if ALLOW_UPDATE
+                    "  Update metadata.\n"
+#endif
+                  , progname);
 }
 
 static const char *const t1 [] = { "path-to-database", "access database metadata", NULL };
@@ -707,19 +967,43 @@ static const char *const q1 [] = { "*","all nodes and attributes", NULL };
 static const char *const q2 [] = { "NAME","a named root node and children", NULL };
 static const char *const q3 [] = { "PATH/NAME","an internal node and children", NULL };
 static const char *const q4 [] = { "<node>@ATTR","a named attribute", NULL };
+#if ALLOW_UPDATE
 static const char *const q5 [] = { "<obj>=VALUE","a simple value assignment where",
                                    "value string is text, and binary",
                                    "values use hex escape codes", NULL };
+#endif
 
-static const char* o1[] = { "try to interpret binary values as unsigned ints", NULL };
+/* static const char* o1[] = { "try to interpret binary values as unsigned ints", NULL }; */
 
-const OptDef opt[] =
-{
-    {"unsigned", NULL, NULL, o1, 0, false, false}
+#define ALIAS_READ_ONLY             "r"
+#define OPTION_READ_ONLY            "read-only"
+static const char* USAGE_READ_ONLY[] = { "operate in read-only mode", NULL };
+
+#define ALIAS_TABLE             "T"
+#define OPTION_TABLE            "table"
+static const char* USAGE_TABLE[] = { "table-name", NULL };
+
+#define ALIAS_UNSIGNED             "u"
+#define OPTION_UNSIGNED            "unsigned"
+static const char* USAGE_UNSIGNED[]
+                                 = { "print numeric values as unsigned", NULL };
+
+const OptDef opt[] = {
+  { OPTION_TABLE    , ALIAS_TABLE    , NULL, USAGE_TABLE    , 1, true , false }
+ ,{ OPTION_UNSIGNED , ALIAS_UNSIGNED , NULL, USAGE_UNSIGNED , 1, false, false }
+#if ALLOW_UPDATE
+ ,{ OPTION_READ_ONLY, ALIAS_READ_ONLY, NULL, USAGE_READ_ONLY, 1, false, false }
+#endif
 };
 
 static const char * const * target_usage [] = { t1, t2, t3, t4 };
-static const char * const * query_usage []  = { q1, q2, q3, q4, q5 };
+static const char * const * query_usage []  =
+{
+    q1, q2, q3, q4
+#if ALLOW_UPDATE
+    , q5
+#endif
+};
 
 
 rc_t CC Usage (const Args * args)
@@ -789,7 +1073,7 @@ rc_t CC Usage (const Args * args)
  */
 rc_t CC KMain ( int argc, char *argv [] )
 {
-    Args * args;
+    Args * args = NULL;
     rc_t rc;
 
     rc = ArgsMakeAndHandle(&args, argc, argv, 1, opt, sizeof(opt) / sizeof(opt[0]));
@@ -802,21 +1086,50 @@ rc_t CC KMain ( int argc, char *argv [] )
             uint32_t pcount;
             int ix;
 
-            if( (rc = ArgsOptionCount(args, opt[0].name, &pcount)) != 0 ) {
+#if ALLOW_UPDATE
+            read_only_arg = false;
+            rc = ArgsOptionCount (args, OPTION_READ_ONLY, &pcount);
+            if (rc) {
+                LOGERR(klogErr, rc,
+                    "Failure to get '" OPTION_READ_ONLY "' argument");
+                break;
+            }
+            read_only_arg = pcount > 0;
+#endif
+
+            rc = ArgsOptionCount (args, OPTION_UNSIGNED, &pcount);
+            if (rc) {
+                LOGERR(klogErr, rc,
+                    "Failure to get '" OPTION_UNSIGNED "' argument");
                 break;
             }
             as_unsigned = pcount > 0;
 
+            rc = ArgsOptionCount (args, OPTION_TABLE, &pcount);
+            if (rc) {
+                LOGERR(klogErr, rc,
+                    "Failure to get '" OPTION_TABLE "' argument");
+                break;
+            }
+            if (pcount) {
+                rc = ArgsOptionValue (args, OPTION_TABLE, 0, &table_arg);
+                if (rc) {
+                    LOGERR(klogErr, rc,
+                        "Failure to get '" OPTION_TABLE "' argument");
+                    break;
+                }
+            }
+
             rc = ArgsParamCount (args, &pcount);
             if (rc)
                 break;
 
             if (pcount == 0)
             {
-                rc = RC (rcExe, rcArgv, rcReading, rcParam, rcInsufficient);
-                LOGERR (klogFatal, rc, "missing database target path and queries" );
+                OUTMSG (( "missing database target path and queries\n" ));
                 MiniUsage (args);
-                break;
+                ArgsWhack(args);
+                exit(EXIT_FAILURE);
             }
 
             rc = KDirectoryNativeDir (&curwd);
@@ -826,9 +1139,12 @@ rc_t CC KMain ( int argc, char *argv [] )
             }
             else
             {
-                const KDBManager * mgr;
-
+                CONST KDBManager * mgr;
+#if ALLOW_UPDATE
+                rc = KDBManagerMakeUpdate (&mgr, curwd);
+#else
                 rc = KDBManagerMakeRead (&mgr, curwd);
+#endif
                 if (rc)
                     LOGERR (klogInt, rc, "Unable to open the database system");
                 else
@@ -840,27 +1156,33 @@ rc_t CC KMain ( int argc, char *argv [] )
                     else
                     {
                         char objpath [ 4096 ];
-                        SRAPath * sra_path;
                         bool found;
                         uint32_t type;
 
                         found = false;
 
-                        rc = SRAPathMake (&sra_path, NULL);
-                        if ( rc == 0 )
+#if ! WINDOWS /* TOOLS_USE_SRAPATH != 0 */
+#warning fix kdbmanagerVPathType to understand accessions
+#if 0
                         {
-                            if ( ! SRAPathTest (sra_path, pc))
+                            SRAPath * sra_path = NULL;
+                            rc = SRAPathMake (&sra_path, NULL);
+                            if ( rc == 0 )
                             {
-                                rc = SRAPathFind (sra_path, pc, objpath, sizeof (objpath));
-                                if (rc == 0)
-                                    found = true;
+                                if ( ! SRAPathTest (sra_path, pc))
+                                {
+                                    rc = SRAPathFind (sra_path, pc, objpath, sizeof (objpath));
+                                    if (rc == 0)
+                                        found = true;
+                                }
+                                SRAPathRelease (sra_path);
+                            }
+                            else if ( GetRCState ( rc ) == rcNotFound && GetRCTarget ( rc ) == rcDylib )
+                            {
+                                rc = 0;
                             }
-                            SRAPathRelease (sra_path);
-                        }
-                        else if ( GetRCState ( rc ) == rcNotFound && GetRCTarget ( rc ) == rcDylib )
-                        {
-                            rc = 0;
                         }
+#endif
 
                         if ( ! found)
                         {
@@ -870,6 +1192,7 @@ rc_t CC KMain ( int argc, char *argv [] )
                             if (rc)
                                 LOGERR (klogFatal, rc, "Unable to resolved target path");
                         }
+#endif
 
 
                         type = KDBManagerVPathType (mgr, objpath, NULL);
@@ -938,6 +1261,9 @@ rc_t CC KMain ( int argc, char *argv [] )
             }
         } while (0);
     }
-    /* done */
+    
+    ArgsWhack(args);
+    args = NULL;
+
     return rc;
 }
