@@ -23,32 +23,41 @@
 * ===========================================================================
 *
 */
+
 #include "vdb-config.vers.h"
 
 #include <kapp/main.h>
 
+#include <vdb/vdb-priv.h> /* VDBManagerListExternalSchemaModules */
+#include <vdb/manager.h> /* VDBManager */
+
 #include <kfg/kfg-priv.h> /* KConfig */
 
-#include <kfs/kfs-priv.h> /* KPathGetCWD */
+#include <vfs/path-priv.h> /* KPathGetCWD */
 #include <kfs/directory.h>
 #include <kfs/file.h>
+#include <kfs/impl.h> /* KDirectoryGetSysDir */
+#include <kfs/kfs-priv.h> /* KSysDirOSPath */
 
-#include <klib/namelist.h>
-#include <klib/text.h> /* strcase_cmp */
 #include <klib/log.h> /* LOGERR */
+#include <klib/namelist.h>
 #include <klib/out.h> /* OUTMSG */
+#include <klib/printf.h> /* string_printf */
 #include <klib/rc.h> /* RC */
+#include <klib/text.h> /* strcase_cmp */
 
+#include <sysalloc.h> /* redefine malloc etc calls */
 #include <os-native.h> /* SHLX */
 
 #include <assert.h>
 #include <errno.h>
-#include <limits.h> /* PATH_MAX */
 #include <stdio.h> /* scanf */
 #include <stdlib.h> /* getenv */
 #include <string.h> /* memset */
 /* #include <unistd.h> access */
 
+#include <limits.h> /* PATH_MAX */
+
 #ifndef PATH_MAX
 #define PATH_MAX 4096
 #endif
@@ -57,7 +66,7 @@
 #define DISP_RC2(rc, name, msg) (void)((rc == 0) ? 0 : \
     PLOGERR(klogInt, (klogInt, rc, \
         "$(name): $(msg)", "name=%s,msg=%s", name, msg)))
-#define DESTRUCT(type, obj) do { rc_t rc2 = type##Release(obj); \
+#define RELEASE(type, obj) do { rc_t rc2 = type##Release(obj); \
     if (rc2 && !rc) { rc = rc2; } obj = NULL; } while (false)
 
 #define ALIAS_ALL    "a"
@@ -80,23 +89,39 @@ static const char* USAGE_ENV[] = { "print shell variables", NULL };
 #define OPTION_FIL   "files"
 static const char* USAGE_FIL[] = { "print loaded files", NULL };
 
+#define ALIAS_IMP    NULL
+#define OPTION_IMP   "import"
+static const char* USAGE_IMP[] = { "import ngc file", NULL };
+
 #define ALIAS_MOD    "m"
 #define OPTION_MOD   "modules"
 static const char* USAGE_MOD[] = { "print external modules", NULL };
 
+#define ALIAS_OUT    "o"
+#define OPTION_OUT   "output"
+static const char* USAGE_OUT[] = { "output type: one of (x n), "
+    "where 'x' is xml (default), 'n' is native", NULL };
+
 #define ALIAS_CFG    "p"
 #define OPTION_CFG   "cfg"
 static const char* USAGE_CFG[] = { "print current configuration", NULL };
 
+#define ALIAS_SET    "s"
+#define OPTION_SET   "set"
+static const char* USAGE_SET[] = { "set configuration node value", NULL };
+
 OptDef Options[] =
-{
+{                                         /* needs_value, required */
       { OPTION_ALL, ALIAS_ALL, NULL, USAGE_ALL, 1, false, false }
     , { OPTION_CFG, ALIAS_CFG, NULL, USAGE_CFG, 1, false, false }
     , { OPTION_DIR, ALIAS_DIR, NULL, USAGE_DIR, 1, false, false }
     , { OPTION_ENV, ALIAS_ENV, NULL, USAGE_ENV, 1, false, false }
     , { OPTION_FIL, ALIAS_FIL, NULL, USAGE_FIL, 1, false, false }
+    , { OPTION_IMP, ALIAS_IMP, NULL, USAGE_IMP, 1, true , false }
     , { OPTION_MOD, ALIAS_MOD, NULL, USAGE_MOD, 1, false, false }
     , { OPTION_NEW, ALIAS_NEW, NULL, USAGE_NEW, 1, false, false }
+    , { OPTION_OUT, ALIAS_OUT, NULL, USAGE_OUT, 1, true , false }
+    , { OPTION_SET, ALIAS_SET, NULL, USAGE_SET, 1, true , false }
 };
 
 rc_t CC UsageSummary (const char * progname) {
@@ -129,6 +154,15 @@ rc_t CC Usage(const Args* args) {
 /*  HelpOptionLine (ALIAS_NEW, OPTION_MOD, NULL, USAGE_NEW); */
     HelpOptionLine (ALIAS_FIL, OPTION_FIL, NULL, USAGE_FIL);
     HelpOptionLine (ALIAS_ENV, OPTION_ENV, NULL, USAGE_ENV);
+    HelpOptionLine (ALIAS_MOD, OPTION_MOD, NULL, USAGE_MOD);
+    KOutMsg ("\n");
+    HelpOptionLine (ALIAS_SET, OPTION_SET, "name=value", USAGE_SET);
+    KOutMsg ("\n");
+    HelpOptionLine (ALIAS_IMP, OPTION_IMP, "ngc-file", USAGE_IMP);
+    KOutMsg ("\n");
+    HelpOptionLine (ALIAS_OUT, OPTION_OUT, "x | n", USAGE_OUT);
+
+    KOutMsg ("\n");
 
     HelpOptionsStandard ();
 
@@ -141,9 +175,9 @@ const char UsageDefaultName[] = "vdb-config";
 
 ver_t CC KAppVersion(void) { return VDB_CONFIG_VERS; }
 
-static rc_t ArgsRelease(Args* self) { return ArgsWhack(self); }
-
-static void Indent(int n) {
+static void Indent(bool xml, int n) {
+    if (!xml)
+    {   return; }
     while (n--)
     {   OUTMSG(("  ")); }
 }
@@ -159,7 +193,9 @@ static rc_t KConfigNodeReadData(const KConfigNode* self,
     assert(*num_read <= blen);
     return rc;
 }
-static rc_t KConfigNodePrintChildNames(const KConfigNode* self,
+
+#define VDB_CONGIG_OUTMSG(args) do { if (xml) { OUTMSG(args); } } while (false)
+static rc_t KConfigNodePrintChildNames(bool xml, const KConfigNode* self,
     const char* name, int indent, const char* aFullpath)
 {
     rc_t rc = 0;
@@ -178,8 +214,9 @@ static rc_t KConfigNodePrintChildNames(const KConfigNode* self,
     if (rc == 0) {
         rc = KConfigNodeReadData(node, buffer, sizeof buffer, &num_read);
         hasData = num_read;
-        if (hasData)
-        { /* OUTMSG(("\n%s = \"%.*s\"\n\n", aFullpath, num_read, buffer)); */ }
+        if (hasData) {
+ /* VDB_CONGIG_OUTMSG(("\n%s = \"%.*s\"\n\n", aFullpath, num_read, buffer)); */
+        }
     }
     if (rc == 0)
     {   rc = KConfigNodeListChild(node, &names); }
@@ -188,16 +225,22 @@ static rc_t KConfigNodePrintChildNames(const KConfigNode* self,
         hasChildren = count;
     }
 
-    Indent(indent);
-    OUTMSG(("<%s", name));
+    Indent(xml, indent);
+    VDB_CONGIG_OUTMSG(("<%s", name));
     if (!hasChildren && !hasData)
-    {   OUTMSG(("/>")); }
+    {   VDB_CONGIG_OUTMSG(("/>\n")); }
     else
-    {   OUTMSG((">")); }
-    if (hasData)
-    {   OUTMSG(("%.*s", (int)num_read, buffer)); }
+    {   VDB_CONGIG_OUTMSG((">")); }
+    if (hasData) {
+        if (xml) {
+            VDB_CONGIG_OUTMSG(("%.*s", (int)num_read, buffer));
+        }
+        else {
+            OUTMSG(("%s = \"%.*s\"\n", aFullpath, (uint32_t)num_read, buffer));
+        }
+    }
     if (hasChildren)
-    {   OUTMSG(("\n"));}
+    {   VDB_CONGIG_OUTMSG(("\n"));}
 
     if (hasChildren) {
         for (i = 0; i < count && rc == 0; ++i) {
@@ -216,19 +259,19 @@ static rc_t KConfigNodePrintChildNames(const KConfigNode* self,
             }
             if (rc == 0) {
                 rc = KConfigNodePrintChildNames
-                    (node, name, indent + 1, fullpath);
+                    (xml, node, name, indent + 1, fullpath);
             }
             free(fullpath);
         }
     }
 
     if (hasChildren)
-    {   Indent(indent); }
+    {   Indent(xml, indent); }
     if (hasChildren || hasData)
-    {   OUTMSG(("</%s>\n",name)); }
+    {   VDB_CONGIG_OUTMSG(("</%s>\n",name)); }
 
-    DESTRUCT(KNamelist, names);
-    DESTRUCT(KConfigNode, node);
+    RELEASE(KNamelist, names);
+    RELEASE(KConfigNode, node);
     return rc;
 }
 
@@ -237,6 +280,13 @@ typedef struct Params {
     uint32_t argsParamIdx;
     uint32_t argsParamCnt;
 
+    bool xml;
+
+    const char* setValue;
+
+    const char* ngc;
+
+    bool modeSetNode;
     bool modeCreate;
     bool modeShowCfg;
     bool modeShowEnv;
@@ -272,7 +322,30 @@ static rc_t ParamsConstruct(int argc, char* argv[], Params* prm) {
             ++count;
         }
 
-        rc = ArgsOptionCount (args, OPTION_CFG, &pcount);
+        prm->xml = true;
+        rc = ArgsOptionCount(args, OPTION_OUT, &pcount);
+        if (rc) {
+            LOGERR(klogErr, rc, "Failure to get '" OPTION_OUT "' argument");
+            break;
+        }
+        if (pcount) {
+            const char* dummy = NULL;
+            rc = ArgsOptionValue(args, OPTION_OUT, 0, &dummy);
+            if (rc) {
+                LOGERR(klogErr, rc, "Failure to get '" OPTION_OUT "' argument");
+                break;
+            }
+            if (!strcmp(dummy, "n")) {
+                prm->xml = false;
+            }
+            else if (strcmp(dummy, "x")) {
+                rc = RC(rcExe, rcArgv, rcParsing, rcParam, rcInvalid);
+                LOGERR(klogErr, rc, "Bad " OPTION_OUT " value");
+                break;
+            }
+        }
+
+        rc = ArgsOptionCount(args, OPTION_CFG, &pcount);
         if (rc) {
             LOGERR(klogErr, rc, "Failure to get '" OPTION_CFG "' argument");
             break;
@@ -284,7 +357,7 @@ static rc_t ParamsConstruct(int argc, char* argv[], Params* prm) {
             }
         }
 
-        rc = ArgsOptionCount (args, OPTION_ENV, &pcount);
+        rc = ArgsOptionCount(args, OPTION_ENV, &pcount);
         if (rc) {
             LOGERR(klogErr, rc, "Failure to get '" OPTION_ENV "' argument");
             break;
@@ -294,7 +367,7 @@ static rc_t ParamsConstruct(int argc, char* argv[], Params* prm) {
             ++count;
         }
 
-        rc = ArgsOptionCount (args, OPTION_FIL, &pcount);
+        rc = ArgsOptionCount(args, OPTION_FIL, &pcount);
         if (rc) {
             LOGERR(klogErr, rc, "Failure to get '" OPTION_FIL "' argument");
             break;
@@ -304,7 +377,20 @@ static rc_t ParamsConstruct(int argc, char* argv[], Params* prm) {
             ++count;
         }
 
-        rc = ArgsOptionCount (args, OPTION_MOD, &pcount);
+        rc = ArgsOptionCount(args, OPTION_IMP, &pcount);
+        if (rc) {
+            LOGERR(klogErr, rc, "Failure to get '" OPTION_IMP "' argument");
+            break;
+        }
+        if (pcount) {
+            rc = ArgsOptionValue(args, OPTION_IMP, 0, &prm->ngc);
+            if (rc) {
+                LOGERR(klogErr, rc, "Failure to get '" OPTION_IMP "' argument");
+                break;
+            }
+        }
+
+        rc = ArgsOptionCount(args, OPTION_MOD, &pcount);
         if (rc) {
             LOGERR(klogErr, rc, "Failure to get '" OPTION_MOD "' argument");
             break;
@@ -314,7 +400,8 @@ static rc_t ParamsConstruct(int argc, char* argv[], Params* prm) {
             ++count;
         }
 
-        rc = ArgsOptionCount (args, OPTION_NEW, &pcount);
+#if 0
+        rc = ArgsOptionCount(args, OPTION_NEW, &pcount);
         if (rc) {
             LOGERR(klogErr, rc, "Failure to get '" OPTION_NEW "' argument");
             break;
@@ -323,8 +410,9 @@ static rc_t ParamsConstruct(int argc, char* argv[], Params* prm) {
             prm->modeCreate = true;
             ++count;
         }
+#endif
 
-        rc = ArgsOptionCount (args, OPTION_DIR, &pcount);
+        rc = ArgsOptionCount(args, OPTION_DIR, &pcount);
         if (rc) {
             LOGERR(klogErr, rc, "Failure to get '" OPTION_DIR "' argument");
             break;
@@ -334,7 +422,29 @@ static rc_t ParamsConstruct(int argc, char* argv[], Params* prm) {
             ++count;
         }
 
-        rc = ArgsOptionCount (args, OPTION_ALL, &pcount);
+        rc = ArgsOptionCount(args, OPTION_SET, &pcount);
+        if (rc) {
+            LOGERR(klogErr, rc, "Failure to get '" OPTION_SET "' argument");
+            break;
+        }
+        if (pcount) {
+            rc = ArgsOptionValue(args, OPTION_SET, 0, &prm->setValue);
+            if (rc == 0) {
+                const char* p = strchr(prm->setValue, '=');
+                if (p == NULL || *(p + 1) == '\0') {
+                    rc = RC(rcExe, rcArgv, rcParsing, rcParam, rcInvalid);
+                    LOGERR(klogErr, rc, "Bad " OPTION_SET " value");
+                    break;
+                }
+                prm->modeSetNode = true;
+                prm->modeCreate = prm->modeShowCfg = prm->modeShowEnv
+                    = prm->modeShowFiles = prm->modeShowLoadPath
+                    = prm->modeShowModules = false;
+                count = 1;
+            }
+        }
+
+        rc = ArgsOptionCount(args, OPTION_ALL, &pcount);
         if (rc) {
             LOGERR(klogErr, rc, "Failure to get '" OPTION_ALL "' argument");
             break;
@@ -342,7 +452,8 @@ static rc_t ParamsConstruct(int argc, char* argv[], Params* prm) {
         if (pcount
             || ( !prm->modeShowCfg && ! prm->modeShowLoadPath
               && !prm->modeShowEnv && !prm->modeShowFiles
-              && !prm->modeShowModules && !prm->modeCreate ))
+              && !prm->modeShowModules && !prm->modeCreate
+              && !prm->modeSetNode && !prm->ngc))
             /* show all by default */
         {
             prm->modeShowCfg = prm->modeShowEnv = prm->modeShowFiles = true;
@@ -374,7 +485,7 @@ static rc_t ParamsGetNextParam(Params* prm, const char** param) {
 static rc_t ParamsDestruct(Params* prm) {
     rc_t rc = 0;
     assert(prm);
-    DESTRUCT(Args, prm->args);
+    RELEASE(Args, prm->args);
     return rc;
 }
 
@@ -386,7 +497,7 @@ rc_t privReadStdinLine(char* buf, size_t bsize, bool destroy)
     static uint64_t pos = 0;
     size_t num_read = 0;
     if (destroy) {
-        DESTRUCT(KFile, std_in);
+        RELEASE(KFile, std_in);
         pos = 0;
         return rc;
     }
@@ -496,7 +607,7 @@ static rc_t CreateConfig(char* argv0) {
         if (home)
         {   def = home; }
         else {
-            rc = KPathGetCWD(cwd, sizeof cwd);
+            rc = KDirectoryResolvePath ( native, true, cwd, sizeof cwd, "." );
             if (rc == 0 && cwd[0])
             {   def = cwd; }
             else
@@ -547,7 +658,7 @@ static rc_t CreateConfig(char* argv0) {
         {   break; }
         rc = KDirectoryOpenDirRead(native, &dir, false, refseq);
         if (rc == 0) {
-            DESTRUCT(KDirectory, dir);
+            RELEASE(KDirectory, dir);
             break;
         }
         else if (GetRCObject(rc) == rcPath
@@ -608,128 +719,116 @@ static rc_t CreateConfig(char* argv0) {
                 pos += num_writ;
             }
         }
-        DESTRUCT(KFile, f);
+        RELEASE(KFile, f);
     }
     free(mod);
     free(wmod);
     free(refseq);
     free(location);
-    DESTRUCT(KDirectory, dir);
-    DESTRUCT(KDirectory, native);
-    DESTRUCT(KFile, std_in);
+    RELEASE(KDirectory, dir);
+    RELEASE(KDirectory, native);
+    RELEASE(KFile, std_in);
     DestroyStdin();
     return rc;
 }
 
-
-#ifndef _STATIC
-static rc_t CC scan_mod_dir(const KDirectory* dir,
-    uint32_t type, const char* name, void* data)
-{
-    rc_t rc = 0;
-    const char ext[] = SHLX;
-
-    assert(data);
-
-    if (strlen(name) > strlen(ext) + 1 &&
-        name[strlen(name) - strlen(ext) - 1] == '.')
-    {
-        char buf[PATH_MAX + 1];
-        rc = KDirectoryResolvePath
-            (dir, true, buf, sizeof buf, "%s/%s", data, name);
-
-        while (rc == 0) {
-            uint32_t type = KDirectoryPathType(dir, buf);
-            if (type & kptAlias) {
-                rc = KDirectoryResolveAlias
-                    (dir, true, buf, sizeof buf, buf);
-                DISP_RC(rc, name);
-            }
-            else if (rc == 0) {
-                if (type == kptNotFound || type == kptBadPath) {
-                    OUTMSG(("%s: %s\n", buf,
-                        type == kptNotFound ? "not found" : "bad path"));
-                }
-                else { OUTMSG(("%s\n", buf)); }
-                break;
-            }
-        }
-    }
-
-    return rc;
-}
-
-static rc_t ShowModDir(const KDirectory* native, char* path) {
-    rc_t rc = 0;
-    const KDirectory* dir = NULL;
-    assert(native && path);
-    rc = KDirectoryOpenDirRead(native, &dir, false, path);
-    DISP_RC(rc, path);
-    if (rc == 0) {
-        rc = KDirectoryVVisit(dir, false, scan_mod_dir, path, ".", NULL);
-    }
-    return rc;
-}
-
-#endif
 static rc_t ShowModules(const KConfig* cfg, const Params* prm) {
     rc_t rc = 0;
 #ifdef _STATIC
     OUTMSG(("<!-- Modules are not used in static build -->\n"));
 #else
-    KDirectory* dir = NULL;
-    const char* paths[] = { "vdb/module/paths", "vdb/wmodule/paths" };
-    int i = 0;
-    assert(cfg);
+    const VDBManager* mgr = NULL;
+    KNamelist* list = NULL;
     OUTMSG(("<!-- Modules -->\n"));
-    for (i = 0; i < sizeof paths / sizeof paths[0] && rc == 0; ++i) {
-        const KConfigNode* node = NULL;
+    rc = VDBManagerMakeRead(&mgr, NULL);
+    DISP_RC(rc, "while calling VDBManagerMakeRead");
+    if (rc == 0) {
+        rc = VDBManagerListExternalSchemaModules(mgr, &list);
+        DISP_RC(rc, "while calling VDBManagerListExternalSchemaModules");
+    }
+    if (rc == 0) {
+        uint32_t count = 0;
+        rc = KNamelistCount(list, &count);
+        DISP_RC(rc, "while calling KNamelistCount "
+            "on VDBManagerListExternalSchemaModules result");
         if (rc == 0) {
-            const char* path = paths[i];
-            rc = KConfigOpenNodeRead(cfg, &node, path);
-            if (rc != 0) {
-                if (GetRCState(rc) == rcNotFound) {
-                    rc = 0;
-                    continue;
-                }
-                else {  DISP_RC(rc, path); }
-            }
-            else {
-                char buf[PATH_MAX + 1];
-                size_t num_read = 0;
-                size_t remaining = 0;
-                rc = KConfigNodeRead(node, 0,
-                    buf, sizeof buf, &num_read, &remaining);
-                assert(remaining == 0);
-                assert(num_read <= sizeof buf);
-                DISP_RC(rc, path);
+            int64_t i = 0;
+            for (i = 0; i < count && rc == 0; ++i) {
+                const char* name = NULL;
+                rc = KNamelistGet(list, i, &name);
+                DISP_RC(rc, "while calling KNamelistGet "
+                    "on VDBManagerListExternalSchemaModules result");
                 if (rc == 0) {
-                    if (num_read < sizeof buf) {
-                        buf[num_read] = '\0';
-                        if (dir == NULL)
-                        {   rc = KDirectoryNativeDir(&dir); }
-                        if (rc == 0) {
-                            OUTMSG(("%s = %s\n", path, buf));
-                            rc = ShowModDir(dir, buf);
-                            if (rc == 0)
-                            {   OUTMSG(("\n")); }
-                        }
-                    }
+                    OUTMSG(("%s\n", name));
                 }
             }
-            DESTRUCT(KConfigNode, node);
         }
     }
-    DESTRUCT(KDirectory, dir);
+    OUTMSG(("\n"));
+    RELEASE(KNamelist, list);
+    RELEASE(VDBManager, mgr);
 #endif
     return rc;
 }
 
+static rc_t SetNode(KConfig* cfg, const Params* prm) {
+    rc_t rc = 0;
+
+    KConfigNode* node = NULL;
+    char* name = NULL;
+    char* val  = NULL;
+
+    assert(cfg && prm && prm->setValue);
+
+    name = strdup(prm->setValue);
+    if (name == NULL)
+    {   return RC(rcExe, rcStorage, rcAllocating, rcMemory, rcExhausted); }
+
+    val = strchr(name, '=');
+    if (val == NULL || *(val + 1) == '\0') {
+        rc_t rc = RC(rcExe, rcArgv, rcParsing, rcParam, rcInvalid);
+        LOGERR(klogErr, rc, "Bad " OPTION_SET " value");
+    }
+
+    if (rc == 0) {
+        *(val++) = '\0';
+
+        rc = KConfigOpenNodeUpdate(cfg, &node, name);
+        if (rc != 0) {
+            PLOGERR(klogErr, (klogErr, rc,
+                "Cannot open node '$(name)' for update", "name=%s", name));
+        }
+    }
+
+    if (rc == 0) {
+        assert(val);
+        rc = KConfigNodeWrite(node, val, strlen(val));
+        if (rc != 0) {
+            PLOGERR(klogErr, (klogErr, rc,
+                "Cannot write value '$(val) to node '$(name)'",
+                "val=%s,name=%s", val, name));
+        }
+    }
+
+    if (rc == 0) {
+        rc = KConfigCommit(cfg);
+        DISP_RC(rc, "while calling KConfigCommit");
+    }
+
+    free(name);
+    name = NULL;
+
+    RELEASE(KConfigNode, node);
+    return rc;
+}
+
 static rc_t ShowConfig(const KConfig* cfg, Params* prm) {
     rc_t rc = 0;
     bool hasAny = false;
     bool hasQuery = false;
+    bool xml = true;
     assert(cfg && prm);
+    xml = prm->xml;
     while (rc == 0) {
         KNamelist* names = NULL;
         const KConfigNode* node = NULL;
@@ -765,9 +864,12 @@ static rc_t ShowConfig(const KConfig* cfg, Params* prm) {
             }
             else {
                 char *c = memrchr(root, '/', len);
-                if (c != NULL)
-                {   nodeName = c + 1; }
-                else { nodeName = root; }
+                if (c != NULL) {
+                    nodeName = c + 1;
+                }
+                else {
+                    nodeName = root;
+                }
             }
             assert(nodeName && nodeName[0]);
             nodeNameL = strlen(nodeName);
@@ -792,9 +894,15 @@ static rc_t ShowConfig(const KConfig* cfg, Params* prm) {
             if (rc == 0 && num_read > 0) {
                 if (prm->showMultiple)
                 {   OUTMSG(("<!-- Configuration node %s -->\n", root)); }
-                OUTMSG(("<%.*s>", nodeNameL, nodeName));
-                OUTMSG(("%.*s", (int)num_read, buf));
-                OUTMSG(("</%.*s>\n", nodeNameL, nodeName));
+                if (xml) {
+                    VDB_CONGIG_OUTMSG(("<%.*s>", nodeNameL, nodeName));
+                    VDB_CONGIG_OUTMSG(("%.*s", (int)num_read, buf));
+                    VDB_CONGIG_OUTMSG(("</%.*s>\n", nodeNameL, nodeName));
+                }
+                else {
+                    OUTMSG(("%.*s = \"%.*s\"\n",
+                        nodeNameL, nodeName, (int)num_read, buf));
+                }
                 hasAny = true;
             }
         }
@@ -803,11 +911,11 @@ static rc_t ShowConfig(const KConfig* cfg, Params* prm) {
                 if (nodeName[0] != '/') {
                     if (prm->showMultiple)
                     {   OUTMSG(("<!-- Configuration node %s -->\n", root)); }
-                    OUTMSG(("<%.*s>\n", nodeNameL, nodeName));
+                    VDB_CONGIG_OUTMSG(("<%.*s>\n", nodeNameL, nodeName));
                 } else {
                     if (prm->showMultiple)
                     {   OUTMSG(("<!-- Current configuration -->\n")); }
-                    OUTMSG(("<Config>\n"));
+                    VDB_CONGIG_OUTMSG(("<Config>\n"));
                 }
                 hasAny = true;
                 ++indent;
@@ -819,22 +927,28 @@ static rc_t ShowConfig(const KConfig* cfg, Params* prm) {
                 if (rc == 0) {
                     char* fullname = NULL;
                     if (strcmp(root, "/") == 0) {
-                        fullname = calloc(1, 1);
+                        fullname = malloc(strlen(name) + 2);
                         if (fullname == NULL) {
                             rc = RC(rcExe,
                                 rcStorage, rcAllocating, rcMemory, rcExhausted);
                         }
+                        sprintf(fullname, "/%s", name);
                     }
                     else {
-                        fullname = strdup(root);
+                        size_t sz = strlen(root) + 2 + strlen(name);
+                        size_t num_writ = 0;
+                        fullname = malloc(sz);
                         if (fullname == NULL) {
                             rc = RC(rcExe,
                                 rcStorage, rcAllocating, rcMemory, rcExhausted);
                         }
+                        rc = string_printf(fullname, sz, &num_writ,
+                            "%s/%s", root, name);
+                        assert(num_writ + 1 == sz);
                     }
                     if (rc == 0) {
                         rc = KConfigNodePrintChildNames
-                            (node, name, indent, fullname);
+                            (xml, node, name, indent, fullname);
                         hasAny = true;
                     }
                     free(fullname);
@@ -842,17 +956,26 @@ static rc_t ShowConfig(const KConfig* cfg, Params* prm) {
                 }
             }
             if (rc == 0) {
-                if (nodeName[0] != '/')
-                {   OUTMSG(("</%.*s>\n", nodeNameL, nodeName)); }
-                else { OUTMSG(("</Config>\n")); }
+                if (nodeName[0] != '/') {
+                    VDB_CONGIG_OUTMSG(("</%.*s>\n", nodeNameL, nodeName));
+                }
+                else {
+                    VDB_CONGIG_OUTMSG(("</Config>\n"));
+                }
             }
         }
 
-        DESTRUCT(KConfigNode, node);
-        DESTRUCT(KNamelist, names);
+        RELEASE(KConfigNode, node);
+        RELEASE(KNamelist, names);
 
-        if (rc == 0 && hasAny)
-        {   OUTMSG(("\n")); }
+        if (rc == 0) {
+            if (hasAny) {
+                OUTMSG(("\n"));
+            }
+            else if (nodeNameL > 0 && nodeName != NULL) {
+                VDB_CONGIG_OUTMSG(("<%.*s/>\n", nodeNameL, nodeName));
+            }
+        }
 
         if (!hasQuery)
         {   break; }
@@ -889,6 +1012,9 @@ static rc_t ShowFiles(const KConfig* cfg, const Params* prm) {
     }
     if (rc == 0 && hasAny)
     {   OUTMSG(("\n")); }
+
+    RELEASE(KNamelist, names);
+
     return rc;
 }
 
@@ -897,7 +1023,8 @@ static void ShowEnv(const Params* prm) {
     const char * env_list [] = {
         "KLIB_CONFIG",
         "VDB_CONFIG",
-        "VDBCONFIG"
+        "VDBCONFIG",
+        "LD_LIBRARY_PATH"
     };
     int i = 0;
 
@@ -924,8 +1051,9 @@ rc_t CC KMain(int argc, char* argv[]) {
     Params prm;
     KConfig* cfg = NULL;
 
-    if (rc == 0)
-    {   rc = ParamsConstruct(argc, argv, &prm); }
+    if (rc == 0) {
+        rc = ParamsConstruct(argc, argv, &prm);
+    }
 
     if (rc == 0) {
         rc = KConfigMake(&cfg, NULL);
@@ -933,34 +1061,72 @@ rc_t CC KMain(int argc, char* argv[]) {
     }
 
     if (rc == 0) {
-        if (prm.modeShowCfg)
-        {   rc = ShowConfig(cfg, &prm); }
+        if (prm.ngc) {
+            const char *newRepoParentPath = NULL;
+            rc = KConfigImportNgc(cfg, prm.ngc, NULL, &newRepoParentPath);
+            if (rc == 0) {
+                rc = KConfigCommit(cfg);
+            }
+            if (rc == 0) {
+#if WINDOWS
+                KDirectory *wd = NULL;
+                char system[MAX_PATH] = "";
+                rc_t rc = KDirectoryNativeDir(&wd);
+                if (rc == 0) {
+                    rc = KDirectoryPosixStringToSystemString(wd,
+                        system, sizeof system, newRepoParentPath);
+                    if (rc == 0) {
+                        newRepoParentPath = system;
+                    }
+                }
+#endif          
+                OUTMSG((
+                    "%s was imported\nThe new protected repository is: %s\n",
+                    prm.ngc, newRepoParentPath));
+            }
+        }
+        else if (prm.modeSetNode) {
+            rc_t rc3 = SetNode(cfg, &prm);
+            if (rc3 != 0 && rc == 0) {
+                rc = rc3;
+            }
+        }
+        if (prm.modeShowCfg) {
+            rc_t rc3 = ShowConfig(cfg, &prm);
+            if (rc3 != 0 && rc == 0) {
+                rc = rc3;
+            }
+        }
         if (prm.modeShowFiles) {
             rc_t rc3 = ShowFiles(cfg, &prm);
-            if (rc3 != 0 && rc == 0)
-            {   rc = rc3; }
+            if (rc3 != 0 && rc == 0) {
+                rc = rc3;
+            }
         }
         if (prm.modeShowModules) {
             rc_t rc3 = ShowModules(cfg, &prm);
-            if (rc3 != 0 && rc == 0)
-            {   rc = rc3; }
+            if (rc3 != 0 && rc == 0) {
+                rc = rc3;
+            }
         }
         if (prm.modeShowLoadPath) {
             const char* path = NULL;
             rc_t rc3 = KConfigGetLoadPath(cfg, &path);
             if (rc3 == 0) {
-                if (path != NULL && path[0])
-                {   OUTMSG(("%s\n", path)); }
+                if (path != NULL && path[0]) {
+                    OUTMSG(("%s\n", path));
+                }
+            }
+            else if (rc == 0) {
+                rc = rc3;
             }
-            else if (rc == 0)
-            {   rc = rc3; }
         }
     }
 
     if (prm.modeShowEnv)
     {   ShowEnv(&prm); }
 
-    DESTRUCT(KConfig, cfg);
+    RELEASE(KConfig, cfg);
 
     if (rc == 0 && prm.modeCreate)
     {   rc = CreateConfig(argv[0]); }
